# -*- coding: utf-8 -*-

# Copyright (c) 2014 Baidu.com, Inc. All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

"""
This module provide billing information and eip status condition.
"""
from enum import Enum


class Billing(object):
    """
    billing information
    """

    def __init__(self, payment_timing=None, billing_method=None, reservation_length=None,
                 reservation_time_unit=None):
        """

        :type payment_timing: string
        :param payment_timing: The pay time of the payment, default value 'Postpaid'

        :type billing_method: string
        :param billing_method: The way of eip charging, default value 'ByBandwidth'

        :type reservation_length: int
        :param reservation_length: purchase length

        :type reservation_time_unit: string
        :param reservation_time_unit: time unit of purchasing，default 'Month'
        """
        self.payment_timing = payment_timing
        self.billing_method = billing_method
        self.reservation_length = reservation_length
        self.reservation_time_unit = reservation_time_unit or 'Month'


class EipStatus(Enum):
    """
    eip status enum query condition.
    """
    AVAILABLE = b'available'
    BINDED = b'binded'
    PAUSED = b'paused'



# Copyright 2014 Baidu, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License") you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

"""
The type of KeySpec
"""

BAIDU_AES_256 = 'BAIDU_AES_256'
AES_128 = 'AES_128'
AES_256 = 'AES_256'
SM1_128 = 'SM1_128'
SM2_256 = 'SM2_256'
SM4_128 = 'SM4_128'
RSA_1024 = 'RSA_1024'
RSA_2048 = 'RSA_2048'
RSA_4096 = 'RSA_4096'



# Copyright (c) 2020 Baidu.com, Inc. All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions

# and limitations under the License.
"""
This module provides a client class for CERT.
"""

from __future__ import unicode_literals

import copy
import json
import logging

from baidubce import bce_base_client
from baidubce import utils
from baidubce.auth import bce_v1_signer
from baidubce.http import bce_http_client
from baidubce.http import handler
from baidubce.http import http_content_types
from baidubce.http import http_headers
from baidubce.http import http_methods
from baidubce.services.cert import cert_model

_logger = logging.getLogger(__name__)


class CertClient(bce_base_client.BceBaseClient):
    """
    CertClient
    """
    prefix = b"/v1/certificate"

    def __init__(self, config=None):
        bce_base_client.BceBaseClient.__init__(self, config)

    @staticmethod
    def _merge_config(self, config):
        if config is None:
            return self.config
        else:
            new_config = copy.copy(self.config)
            new_config.merge_non_none_values(config)
            return new_config

    def _send_request(
            self, http_method, path,
            body=None, headers=None, params=None,
            config=None,
            body_parser=None):
        config = self._merge_config(self, config)
        if body_parser is None:
            body_parser = handler.parse_json
        headers = headers or {}
        headers[http_headers.CONTENT_TYPE] = http_content_types.JSON

        return bce_http_client.send_request(
            config, bce_v1_signer.sign, [handler.parse_error, body_parser],
            http_method, utils.append_uri(CertClient.prefix, path), body, headers, params)

    def create_cert(self, cert_create_request, config=None):
        """
        create certificate

        :param cert_create_request: certificate base informations
        :type cert_create_request: cert_model.CertCreateRequest

        :param config: None
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype: baidubce.bce_response.BceResponse
        """
        return self._send_request(
            http_methods.POST, '', body=json.dumps(cert_create_request.__dict__),
            config=config)

    def list_user_certs(self, config=None):
        """
        list user's certificates

        :param config: None
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype: baidubce.bce_response.BceResponse
        """
        return self._send_request(
            http_methods.GET, '',
            config=config)

    def get_cert_info(self, cert_id, config=None):
        """
        get a certificate information by id

        :param cert_id: certificate id
        :type cert_id: string

        :param config: None
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype: baidubce.bce_response.BceResponse
        """
        return self._send_request(
            http_methods.GET, '/' + cert_id,
            config=config)

    def delete_cert(self, cert_id, config=None):
        """
        delete a certificate by id
        :param cert_id: certificate id
        :type cert_id: string
        :param config: None
        :type config: baidubce.BceClientConfiguration
        :return:
        :rtype: baidubce.bce_response.BceResponse
        """
        return self._send_request(
            http_methods.DELETE, '/' + cert_id,
            config=config)

    def replace_cert(self, cert_id, cert_create_request, config=None):
        """
        delete a certificate by id

        :param cert_id: certificate id
        :type cert_id: string

        :param cert_create_request: certificate base informations
        :type cert_create_request: cert_model.CertCreateRequest

        :param config: None
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype: baidubce.bce_response.BceResponse
        """
        return self._send_request(
            http_methods.PUT, '/' + cert_id,
            body=json.dumps(cert_create_request.__dict__),
            params={'certData': ''},
            config=config)



# Copyright 2014 Baidu, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

"""
This module provides a client class for BES.
"""

import copy
import json
import logging
import sys

from baidubce import bce_base_client
from baidubce import compat
from baidubce.auth import bce_v1_signer
from baidubce.http import bce_http_client
from baidubce.http import handler
from baidubce.http import http_methods
from baidubce.services.bes import bes_model
from baidubce.utils import required

_logger = logging.getLogger(__name__)

if sys.version_info[0] == 2:
    value_type = (str, unicode)
else:
    value_type = (str, bytes)


class BesClient(bce_base_client.BceBaseClient):
    """
    Bes sdk client
    """
    prefix = b'/api/bes/cluster'

    def __init__(self, config=None):
        bce_base_client.BceBaseClient.__init__(self, config)

    @required(name=value_type,
              password=value_type,
              modules=list,
              version=value_type,
              is_old_package=bool,
              available_zone=value_type,
              security_group_id=value_type,
              subnet_uuid=value_type,
              vpc_id=value_type,
              billing=bes_model.Billing)
    def create_cluster(self,
                       name,
                       password,
                       modules,
                       version,
                       is_old_package,
                       available_zone,
                       security_group_id,
                       subnet_uuid,
                       vpc_id,
                       billing,
                       client_token=None):
        """
        Create cluster

        :param name: The parameter to specify es cluster name.
        :type name: string

        :param password: The parameter to specify password for manage cluster.
        :type password: string

        :param modules: The parameter to specify modules for cluster.
        :type modules: array

        :param version: The parameter to specify es cluster version.
        :type version: string

        :param is_old_package: The parameter to specify is old package
        :type is_old_package: bool

        :param available_zone: he parameter to specify security zone.
        :type available_zone: string

        :param security_group_id: The parameter to specify id of the securityGroup.
        :type security_group_id: string

        :param subnet_uuid: The parameter to specify id of the subnet.
        :type subnet_uuid: string

        :param vpc_id: The parameter to specify id of the vpc.
        :type vpc_id: string

        :param billing: The parameter to specify id of billing info.
        :type billing: xxx

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = b'/create'
        params = None
        # if client_token is None:
        #     params['clientToken'] = generate_client_token()
        # else:
        #     params['clientToken'] = client_token

        if client_token is not None:
            params = {
                'clientToken': client_token
            }
        module_json_array = []
        for module in modules:
            module_json_array.append(module.__dict__)
        body = {
            'name': name,
            'password': password,
            'modules': module_json_array,
            'version': version,
            'isOldPackage': is_old_package,
            'availableZone': available_zone,
            'securityGroupId': security_group_id,
            'subnetUuid': subnet_uuid,
            'vpcId': vpc_id,
            'billing': billing.__dict__
        }
        region = self.config.region
        headers = {b'x-Region': region,
                   b'content-type': b'application/json;charset=UTF-8'}
        return self._send_request(http_methods.POST, path, params=params, body=json.dumps(body), headers=headers)

    @required(name=value_type,
              payment_type=value_type,
              cluster_id=value_type,
              region=value_type,
              modules=list)
    def resize_cluster(self,
                       name,
                       payment_type,
                       cluster_id,
                       region,
                       modules,
                       client_token=None):
        """
        resize cluster

        :param name: The parameter to specify es cluster name.
        :type name: string

        :param payment_type: The parameter to specify mode of payment.
        :type payment_type: string

        :param cluster_id: The parameter to specify cluster id.
        :type cluster_id: array

        :param region: The parameter to specify region.
        :type region: string

        :param modules: The parameter to specify the type of es cluster node resource.
        :type modules: list

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = b'/resize'
        params = {
            'orderType': "RESIZE"
        }
        # if client_token is None:
        #     params['clientToken'] = generate_client_token()
        # else:
        #     params['clientToken'] = client_token

        # if client_token is not None:
        #     params = {
        #         'clientToken': client_token
        #     }
        module_json_array = []
        for module in modules:
            module_json_array.append(module.__dict__)
        body = {
            'name': name,
            'paymentType': payment_type,
            'modules': module_json_array,
            'clusterId': cluster_id,
            'region': region
        }
        region = self.config.region
        headers = {b'x-Region': region,
                   b'content-type': b'application/json;charset=UTF-8'}
        return self._send_request(http_methods.POST, path, params=params, body=json.dumps(body), headers=headers)

    @required(page_no=int,
              page_size=int)
    def get_cluster_list(self,
                         page_no,
                         page_size,
                         client_token=None):
        """
        get es cluster list
        :param page_no: The parameter to specify cluster list pageNo.
        :type page_no: int

        :param page_size: The parameter to specify cluster list pageSize.
        :type page_size: int

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = b'/list'
        params = None
        # if client_token is None:
        #     params['clientToken'] = generate_client_token()
        # else:
        #     params['clientToken'] = client_token

        if client_token is not None:
            params = {
                'clientToken': client_token
            }
        body = {
            'pageNo': page_no,
            'pageSize': page_size
        }
        region = self.config.region
        headers = {b'x-Region': region,
                   b'content-type': b'application/json;charset=UTF-8'}
        return self._send_request(http_methods.POST, path, params=params, body=json.dumps(body), headers=headers)

    @required(cluster_id=value_type)
    def get_cluster_detail(self,
                           cluster_id,
                           client_token=None):
        """
        get cluster detail info

        :param cluster_id: The parameter to specify cluster id.
        :type cluster_id: string

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = b'/detail'
        params = None
        # if client_token is None:
        #     params['clientToken'] = generate_client_token()
        # else:
        #     params['clientToken'] = client_token

        if client_token is not None:
            params = {
                'clientToken': client_token
            }
        body = {
            'clusterId': cluster_id
        }
        region = self.config.region
        headers = {b'x-Region': region,
                   b'content-type': b'application/json;charset=UTF-8'}
        return self._send_request(http_methods.POST, path, params=params, body=json.dumps(body), headers=headers)

    @required(cluster_id=value_type)
    def start_cluster(self,
                      cluster_id,
                      client_token=None):
        """
        start es cluster

        :param cluster_id: The parameter to specify cluster id.
        :type cluster_id: string

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = b'/start'
        params = None
        # if client_token is None:
        #     params['clientToken'] = generate_client_token()
        # else:
        #     params['clientToken'] = client_token

        if client_token is not None:
            params = {
                'clientToken': client_token
            }
        body = {
            'clusterId': cluster_id
        }
        region = self.config.region
        headers = {b'x-Region': region,
                   b'content-type': b'application/json;charset=UTF-8'}
        return self._send_request(http_methods.POST, path, params=params, body=json.dumps(body), headers=headers)

    @required(cluster_id=value_type)
    def stop_cluster(self,
                     cluster_id,
                     client_token=None):
        """
        stop es cluster

        :param cluster_id: The parameter to specify cluster id.
        :type cluster_id: string

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = b'/stop'
        params = None
        # if client_token is None:
        #     params['clientToken'] = generate_client_token()
        # else:
        #     params['clientToken'] = client_token

        if client_token is not None:
            params = {
                'clientToken': client_token
            }
        body = {
            'clusterId': cluster_id
        }
        region = self.config.region
        headers = {b'x-Region': region,
                   b'content-type': b'application/json;charset=UTF-8'}
        return self._send_request(http_methods.POST, path, params=params, body=json.dumps(body), headers=headers)

    @required(cluster_id=value_type)
    def delete_cluster(self,
                       cluster_id,
                       client_token=None):
        """
        delete es cluster

        :param cluster_id: The parameter to specify cluster id.
        :type cluster_id: string

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = b'/delete'
        params = None
        # if client_token is None:
        #     params['clientToken'] = generate_client_token()
        # else:
        #     params['clientToken'] = client_token

        if client_token is not None:
            params = {
                'clientToken': client_token
            }
        body = {
            'clusterId': cluster_id
        }
        region = self.config.region
        headers = {b'x-Region': region,
                   b'content-type': b'application/json;charset=UTF-8'}
        return self._send_request(http_methods.POST, path, params=params, body=json.dumps(body), headers=headers)

    @required(cluster_id=value_type,
              instance_id=value_type)
    def start_instance(self,
                       cluster_id,
                       instance_id,
                       client_token=None):
        """
        start instance of es cluster

        :param cluster_id: The parameter to specify cluster id.
        :type cluster_id: string

        :param instance_id: The parameter to specify cluster id.
        :type instance_id: string

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = b'/instance/start'
        params = None
        # if client_token is None:
        #     params['clientToken'] = generate_client_token()
        # else:
        #     params['clientToken'] = client_token

        if client_token is not None:
            params = {
                'clientToken': client_token
            }
        body = {
            'clusterId': cluster_id,
            'instanceId': instance_id,
        }
        region = self.config.region
        headers = {b'x-Region': region,
                   b'content-type': b'application/json;charset=UTF-8'}
        return self._send_request(http_methods.POST, path, params=params, body=json.dumps(body), headers=headers)

    @required(cluster_id=value_type,
              instance_id=value_type)
    def stop_instance(self,
                      cluster_id,
                      instance_id,
                      client_token=None):
        """
        stop instance of es cluster

        :param cluster_id: The parameter to specify cluster id.
        :type cluster_id: string

        :param instance_id: The parameter to specify cluster id.
        :type instance_id: string

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = b'/instance/stop'
        params = None
        # if client_token is None:
        #     params['clientToken'] = generate_client_token()
        # else:
        #     params['clientToken'] = client_token

        if client_token is not None:
            params = {
                'clientToken': client_token
            }
        body = {
            'clusterId': cluster_id,
            'instanceId': instance_id,
        }
        region = self.config.region
        headers = {b'x-Region': region,
                   b'content-type': b'application/json;charset=UTF-8'}
        return self._send_request(http_methods.POST, path, params=params, body=json.dumps(body), headers=headers)

    @required(order=value_type,
              order_by=value_type,
              page_no=int,
              page_size=int,
              days_to_expiration=int)
    def get_renew_list(self,
                       order,
                       order_by,
                       page_no,
                       page_size,
                       days_to_expiration,
                       client_token=None):
        """
        get es cluster renew list

        :param order: The parameter to specify order rule.
        :type order: string

        :param order_by: The parameter to specify order field.
        :type order_by: string

        :param page_no: The parameter to specify renew cluster list pageNo.
        :type page_no: int

        :param page_size: The parameter to specify renew cluster list pageSize.
        :type page_size: int

        :param days_to_expiration: The parameter to specify how many days expire.
        :type days_to_expiration: int

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = b'/renew/list'
        params = None

        if client_token is not None:
            params = {
                'clientToken': client_token
            }
        body = {
            'pageNo': page_no,
            'pageSize': page_size,
            'daysToExpiration': days_to_expiration,
            'order': order,
            'orderBy': order_by
        }
        region = self.config.region
        headers = {b'x-Region': region,
                   b'content-type': b'application/json;charset=UTF-8'}
        return self._send_request(http_methods.POST, path, params=params, body=json.dumps(body), headers=headers)

    @required(order=value_type,
              cluster_id=value_type,
              time=int)
    def renew_cluster(self,
                      cluster_id,
                      time,
                      client_token=None):
        """
        renew es cluster

        :param cluster_id: The parameter to specify order cluster id.
        :type cluster_id: string

        :param time: The parameter to specify renew time, unit month.
        :type time: int

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = b'/renew'
        params = None
        params = {
            'orderType': "RENEW"
        }
        if client_token is not None:
            params['clientToken'] = client_token
        body = {
            'serviceType': 'BES',
            'time': time,
            'clusterId': cluster_id
        }
        region = self.config.region
        headers = {b'x-Region': region,
                   b'content-type': b'application/json;charset=UTF-8'}
        return self._send_request(http_methods.POST, path, params=params, body=json.dumps(body), headers=headers)

    @required(cluster_ids=list,
              user_id=value_type,
              region=value_type,
              service_type=value_type,
              service_ids=list,
              renew_time_unit=value_type,
              renew_time=int)
    def create_auto_renew_rule(self,
                               cluster_ids,
                               user_id,
                               region,
                               renew_time_unit,
                               renew_time,
                               client_token=None):
        """
        create cluster auto renew rule

        :param cluster_ids: The parameter to cluster id list.
        :type cluster_ids: list

        :param user_id: The parameter to specify account id.
        :type user_id: string

        :param region: The parameter to specify region.
        :type region: string

        :param renew_time_unit: The parameter to specify renew time unit.
        :type renew_time_unit: string

        :param renew_time: The parameter to specify renew time.
        :type renew_time: int

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = b'/auto_renew_rule/create'

        params = None
        if client_token is not None:
            params = {
                'clientToken': client_token
            }
        body = {
            'clusterIds': cluster_ids,
            'userId': user_id,
            'region': region,
            'serviceType': 'BES',
            'renewTimeUnit': renew_time_unit,
            "renewTime": renew_time
        }
        region = self.config.region
        headers = {b'x-Region': region,
                   b'content-type': b'application/json;charset=UTF-8'}
        return self._send_request(http_methods.POST, path, params=params, body=json.dumps(body), headers=headers)

    def get_auto_renew_rule_list(self,
                                 client_token=None):
        """
        get cluster auto renew rule list

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = b'/auto_renew_rule/list'
        params = None
        if client_token is not None:
            params = {
                'clientToken': client_token
            }
        body = {
            'serviceType': 'BES',
        }
        region = self.config.region
        headers = {b'x-Region': region,
                   b'content-type': b'application/json;charset=UTF-8'}
        return self._send_request(http_methods.POST, path, params=params, body=json.dumps(body), headers=headers)

    @required(cluster_id=value_type,
              account_id=value_type,
              renew_time_unit=value_type,
              renew_time=int)
    def update_auto_renew_rule(self,
                               cluster_id,
                               renew_time_unit,
                               renew_time,
                               client_token=None):
        """
        update cluster auto renew rule

        :param cluster_id: The parameter to cluster id list.
        :type cluster_id: string

        :param renew_time_unit: The parameter to specify renew time unit.
        :type renew_time_unit: string

        :param renew_time: The parameter to specify renew time.
        :type renew_time: int

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = b'/auto_renew_rule/update'
        params = None
        if client_token is not None:
            params = {
                'clientToken': client_token
            }
        body = {
            'clusterId': cluster_id,
            'renewTimeUnit': renew_time_unit,
            "renewTime": renew_time
        }
        region = self.config.region
        headers = {b'x-Region': region,
                   b'content-type': b'application/json;charset=UTF-8'}
        return self._send_request(http_methods.POST, path, params=params, body=json.dumps(body), headers=headers)

    @required(cluster_id=value_type)
    def delete_auto_renew_rule(self,
                               cluster_id,
                               client_token=None):
        """
        delete cluster auto renew rule

        :param cluster_id: The parameter to cluster id list.
        :type cluster_id: string

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = b'/auto_renew_rule/delete'
        params = None
        if client_token is not None:
            params = {
                'clientToken': client_token
            }
        body = {
            'clusterId': cluster_id
        }
        region = self.config.region
        headers = {b'x-Region': region,
                   b'content-type': b'application/json;charset=UTF-8'}
        return self._send_request(http_methods.POST, path, params=params, body=json.dumps(body), headers=headers)

    def _merge_config(self, config=None):
        if config is None:
            return self.config
        else:
            new_config = copy.copy(self.config)
            new_config.merge_non_none_values(config)
            return new_config

    def _send_request(self, http_method, path,
                      body=None, headers=None, params=None,
                      config=None, body_parser=None):
        config = self._merge_config(config)
        if body_parser is None:
            body_parser = handler.parse_json

        config.endpoint = compat.convert_to_bytes(config.endpoint)
        # return bce_http_client.send_request(
        #     config, sign_wrapper([b'host', b'x-bce-date']), [handler.parse_error, body_parser],
        #     http_method, compat.convert_to_bytes(BesClient.prefix + path), body, headers, params)

        return bce_http_client.send_request(
            config, bce_v1_signer.sign, [handler.parse_error, body_parser],
            http_method, BesClient.prefix + path, body, headers, params)



# -*- coding: utf-8 -*-

"""
This module provide billing information and IkeConfig and IPSec
"""


class Billing(object):
    """
    billing information
    """

    def __init__(self, payment_timing=None, billing_method=None, reservation_length=None,
                 reservation_time_unit=None):
        """
        :type payment_timing: string
        :param payment_timing: 'Prepaid'  'Postpaid'

        :type billing_method: string
        :param billing_method: 'ByTraffic' 'ByBandwidt'

        :type reservation_length: int
        :param reservation_length: purchase length

        :type reservation_time_unit: string
        :param reservation_time_unit: time unit of purchasing，currently only supports monthly
        """
        self.payment_timing = payment_timing
        self.billing_method = billing_method
        self.reservation_length = reservation_length or 1
        self.reservation_time_unit = reservation_time_unit or 'Month'


class IkeConfig(object):
    """
    IKE Configuration example
    """

    def __init__(self, ike_version=None, ike_mode=None, ike_enc_alg=None, ike_auth_alg=None, ike_pfs=None,
                 ike_lifeTime=None):
        """
        :type ike_version: string
        :param ike_version: Version, value range ：v1/v2

        :type ike_mode: string
        :param ike_mode: Negotiation mode, value range ：main/aggressive

        :type ike_enc_alg: string
        :param ike_enc_alg: Encryption algorithm, value range ：aes/aes192/aes256/3des

        :type ike_auth_alg: string
        :param ike_auth_alg: Authentication algorithm, value range ：sha1/md5

        :type ike_pfs: string
        :param ike_pfs: DH Grouping, value range ：group2/group5/group14/group24

        :type ike_lifeTime: string
        :param ike_lifeTime: SA Life cycle, value range ：60-86400
        """
        self.ike_version = ike_version
        self.ike_mode = ike_mode
        self.ike_enc_alg = ike_enc_alg
        self.ike_auth_alg = ike_auth_alg
        self.ike_pfs = ike_pfs
        self.ike_lifeTime = ike_lifeTime


class IpsecConfig(object):
    """
    IPSec Configuration example
    """
    def __init__(self, ipsec_enc_alg=None, ipsec_auth_alg=None, ipsec_pfs=None, ipsec_lifetime=None):
        """
        :type ipsec_enc_alg: string
        :param ipsec_enc_alg: Encryption algorithm, value range ：aes/aes192/aes256/3des

        :type ipsec_auth_alg: string
        :param ipsec_auth_alg: Authentication algorithm, value range ：sha1/md5

        :type ipsec_pfs: string
        :param ipsec_pfs: group2/group5/group14/group24

        :type ipsec_lifetime: string
        :param ipsec_lifetime: SA Life cycle, value range ：180-86400
        """
        self.ipsec_enc_alg = ipsec_enc_alg
        self.ipsec_auth_alg = ipsec_auth_alg
        self.ipsec_pfs = ipsec_pfs
        self.ipsec_lifetime = ipsec_lifetime


# -*- coding: utf-8 -*-

# Copyright (c) 2021 Baidu.com, Inc. All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy
#  of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions
# and limitations under the License.

"""
This module provides a client class for EIP TP.
"""

import copy
import json
import logging
import uuid

from baidubce import utils
from baidubce.auth import bce_v1_signer
from baidubce.bce_base_client import BceBaseClient
from baidubce.http import bce_http_client
from baidubce.http import handler
from baidubce.http import http_methods
from baidubce.utils import required

_logger = logging.getLogger(__name__)


class EipTpClient(BceBaseClient):
    """
    EIP_TP sdk client
    """
    version = b'/v1'
    prefix = b'/eiptp'

    def __init__(self, config=None):
        """
        :type config: baidubce.BceClientConfiguration
        :param config:
        """
        BceBaseClient.__init__(self, config)

    @required(reservation_length=int,
              capacity=str)
    def create_eip_tp(self, reservation_length, capacity, deduct_policy=None, package_type=None,
                      client_token=None, config=None):
        """
        Create an eip_tp with the specified options.

        :type reservation_length: int
        :param reservation_length: the reservation length of the eip_tp including 1, 6 and 12 months.

        :type capacity: string
        :param capacity: the capacity of the eip_tp.
        When reservationLength = 1 => capacity: {"10G"/"50G"/"100G"/"500G"/"1T"/"5T"/"10T"/"50T"}
        When reservationLength = 6 => capacity: {"60G"/"300G"/"600G"/"3T"/"6T"/"30T"/"60T"/"300T"}
        When reservationLength = 12 => capacity: {"1T"/"10T"/"50T"/"100T"/"500T"/"1P"}

        :type deduct_policy: string
        :param deduct_policy: the deduct policy of the eip_tp including 'FullTimeDurationPackage'
        and 'TimeDurationPackage'.
        The default deduct policy is 'FullTimeDurationPackage', the optional parameter.

        :type package_type: string
        :param package_type: the eip_tp package type.
        The default package type is 'WebOutBytes', the optional parameter.

        :type client_token: string
        :param client_token: if the clientToken is not specified by the user,
         a random string generated by default algorithm will be used.

        :type config: baidubce.BceClientConfiguration
        :param config:

        :return: created eip_tp id, for example,{"id":"tp-xxxxxxxxxx"}
        """
        body = {
            'reservationLength': reservation_length,
            'capacity': capacity
        }
        if deduct_policy is not None:
            body['deductPolicy'] = deduct_policy
        if package_type is not None:
            body['packageType'] = package_type
        path = self._get_path()
        if client_token is None:
            client_token = self._generate_default_client_token()
        params = {
            b'clientToken': client_token
        }
        return self._send_request(http_methods.POST, path,
                                  body=json.dumps(body), params=params,
                                  config=config)

    @required(id=str)
    def get_eip_tp_detail(self, id, config=None):
        """
        get the eip_tp's detail owned by the authenticated user by the passed eip_tp_id.

        :type id: string
        :param id: eip_tp's id.

        :type config: baidubce.BceClientConfiguration
        :param config:

        :return: detail of eip_tp, for example:
                {
                  "id":"tp-87V5cnkwqO",
                  "deductPolicy":"TimeDurationPackage",
                  "packageType":"WebOutBytes",
                  "status":"RUNNING",
                  "capacity": 10737418240,
                  "usedCapacity": 0,
                  "createTime":"2021-04-10T11:40:57Z",
                  "activeTime": "2021-04-10T11:41:16Z",
                  "expireTime:" "2021-05-10T11:41:16Z"
                }
        """

        path = utils.append_uri(self._get_path(), id)
        return self._send_request(http_methods.GET, path, params=None,
                                  config=config)

    def list_eip_tps(self, id=None, deduct_policy=None, status=None,
                     marker=None, max_keys=1000, config=None):
        """
        get a list of eip_tp owned by the authenticated filtered by specific conditions.

        :type id: string
        :param id: eip_tp's id, the optional parameter.

        :type deduct_policy: string
        :param deduct_policy: eip_tp's deduct policy, 'FullTimeDurationPackage' or 'TimeDurationPackage', the optional parameter.

        :type status: string
        :param status: eip_tp's status, 'RUNNING', 'EXPIRED' or 'USED_UP', the optional parameter.

        :type marker: string
        :param marker: The optional parameter marker specified in the original
         request to specify where in the results to begin listing.

        :type max_keys: int
        :param max_keys: The optional parameter to specifies the max number
         of list result to return. The default value is 1000.

        :type config: baidubce.BceClientConfiguration
        :param config:

        :return: list of eip_tp model, for example:
                {
                    "marker": "tp-87V5cnkwqO",
                    "maxKeys": 1,
                    "nextMarker": "tp-Qn65tYXAx3",
                    "isTruncated": true,
                    "packageList": [
                        {
                           "id":"tp-87V5cnkwqO",
                           "deductPolicy":"TimeDurationPackage",
                           "packageType":"WebOutBytes",
                           "status":"RUNNING",
                           "capacity": 10737418240,
                           "usedCapacity": 0,
                           "createTime":"2021-04-10T11:40:57Z",
                           "activeTime": "2021-04-10T11:41:16Z",
                           "expireTime:" "2021-05-10T11:41:16Z"
                        }
                    ]
                }
       """

        path = self._get_path()
        params = {}
        if id is not None:
            params[b'id'] = id
        if deduct_policy is not None:
            params[b'deductPolicy'] = deduct_policy
        if status is not None:
            params[b'status'] = status
        if marker is not None:
            params[b'marker'] = marker
        if max_keys is not None:
            params[b'maxKeys'] = max_keys
        return self._send_request(http_methods.GET, path, params=params,
                                  config=config)

    @staticmethod
    def _generate_default_client_token():
        """
        default client token by uuid1
        """
        return uuid.uuid1()

    @staticmethod
    def _get_path(prefix=None):
        """
        :type prefix: string
        :param prefix: path prefix
        """
        if prefix is None:
            prefix = EipTpClient.prefix
        return utils.append_uri(EipTpClient.version, prefix)

    def _merge_config(self, config):
        """
        :type config: baidubce.BceClientConfiguration
        :param config:
        :return:
        """
        if config is None:
            return self.config
        else:
            new_config = copy.copy(self.config)
            new_config.merge_non_none_values(config)
            return new_config

    def _send_request(self, http_method, path, body=None, headers=None,
                      params=None, config=None, body_parser=None):
        """
        :param http_method:
        :param path:
        :param body:
        :param headers:
        :param params:

        :type config: baidubce.BceClientConfiguration
        :param config:

        :param body_parser:

        :return: baidubce.BceResponse
        """
        config = self._merge_config(config)
        if body_parser is None:
            body_parser = handler.parse_json

        if headers is None:
            headers = {b'Accept': b'*/*',
                       b'Content-Type': b'application/json;charset=utf-8'}

        return bce_http_client.send_request(config, bce_v1_signer.sign,
                                            [handler.parse_error, body_parser],
                                            http_method, path, body, headers,
                                            params)



# Copyright 2014 Baidu, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

"""
This module defines some constants for BOS
"""

MAX_PUT_OBJECT_LENGTH = 5 * 1024 * 1024 * 1024
MAX_APPEND_OBJECT_LENGTH = 5 * 1024 * 1024 * 1024
MAX_USER_METADATA_SIZE = 2 * 1024
MIN_PART_NUMBER = 1
MAX_PART_NUMBER = 10000
URL_PREFIX = b"/"



#! usr/bin/python
# -*-coding:utf-8 -*-
# Copyright 2014 Baidu, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions
# and limitations under the License.


"""
This module defines some Response classes for BTS
"""
from baidubce.bce_response import BceResponse
from json import JSONEncoder


class Billing(object):
    """
	This class define billing.
	param: paymentTiming:
		The pay time of the payment,
	param: reservationLength:
		The duration to buy in specified time unit,
	param: reservationTimeUnit:
		The time unit to specify the duration ,only "Month" can be used now.
	"""

    def __init__(self, pay_method='Prepaid', reservationLength=1, reservationTimeUnit='Month'):
        self.paymentTiming = pay_method
        self.reservation = {
            'reservationLength': reservationLength,
            'reservationTimeUnit': reservationTimeUnit
        }

    def get_pay_method(self):
        """
            get instance current pay_method:Prepaid/Postpaid
        """
        return self.paymentTiming


class SubnetMap(object):
    """
    SubnetMap:contains zoneName and subnetId
    """

    def __init__(self, zone_name, subnet_id):
        super(SubnetMap, self).__init__()
        self.zone_name = str(zone_name)
        self.subnet_id = str(subnet_id)


class CreateInstanceResponse(BceResponse):
    """
    Create Instance Response
    """

    def __init__(self, bce_response):
        super(CreateInstanceResponse, self).__init__()
        self.instance_ids = bce_response.instance_ids


class ListInstanceResponse(BceResponse):
    """
    List Instance Response
    """

    def __init__(self, bce_response):
        super(ListInstanceResponse, self).__init__()
        self.max_keys = bce_response.max_keys
        self.marker = str(bce_response.marker)
        self.next_marker = str(bce_response.next_marker)
        self.is_truncated = bce_response.is_truncated
        self.instances = bce_response.instances


class GetInstanceResponse(BceResponse):
    """
    Get Instance Response
    """

    def __init__(self, bce_response):
        super(GetInstanceResponse, self).__init__()
        self.instance_id = str(bce_response.instance_id)
        self.instance_name = str(bce_response.instance_name)
        self.instance_status = str(bce_response.instance_status)
        self.cluster_type = str(bce_response.cluster_type)
        self.engine = str(bce_response.engine)
        self.engine_version = str(bce_response.engine_version)
        self.vnet_ip = str(bce_response.vnet_ip)
        self.port = str(bce_response.port)
        self.instance_create_time = bce_response.instance_create_time
        self.instance_expire_time = bce_response.instance_expire_time
        self.capacity = bce_response.capacity
        self.used_capacity = bce_response.used_capacity
        self.payment_timing = str(bce_response.payment_timing)
        self.zone_names = bce_response.zone_names
        self.vpc_id = str(bce_response.vpc_id)
        self.subnets = bce_response.subnets
        self.auto_renew = bce_response.auto_renew
        self.shard_num = bce_response.shard_num
        self.store_type = bce_response.store_type


class ListAvailableZoneResponse(BceResponse):
    """
    List available zone.
    """

    def __init__(self, bce_response):
        super(ListAvailableZoneResponse, self).__init__()
        self.zones = bce_response.zones


class ListSubnetResponse(BceResponse):
    """
    List available zone.
    """

    def __init__(self, bce_response):
        super(ListSubnetResponse, self).__init__()
        self.subnets = bce_response.subnets
        if self.subnets is None:
            self.subnets = []


class ListNodeTypeResponse(BceResponse):
    """
    List nodetypes for scs.
    """

    def __init__(self, bce_response):
        super(ListNodeTypeResponse, self).__init__()
        self.default_node_type_list = bce_response.default_node_type_list
        self.cluster_node_type_list = bce_response.cluster_node_type_list
        self.hsdb_node_type_list = bce_response.hsdb_node_type_list


class NodeType(object):
    """
    NodeType model
    """

    def __init__(self, obj):
        super(NodeType, self).__init__()
        self.node_type = str(obj.node_type)
        self.cpu_num = int(obj.cpu_num)
        self.instance_flavor = int(obj.instance_flavor)
        self.network_throughput_in_gbps = float(obj.network_throughput_in_gbps)
        self.peak_qps = int(obj.peak_qps)
        self.max_connections = int(obj.max_connections)
        self.allowed_node_num_list = obj.allowed_node_num_list
        self.allowed_replication_num_list = obj.allowed_replication_num_list


class Tag(object):
    """
    Tag model
    """

    def __init__(self, key, value):
        super(Tag, self).__init__()
        self.tag_key = str(key)
        self.tag_value = str(value)

    def __repr__(self):
        return repr((self.tag_key, self.tag_value))


class ListSecurityIpResponse(BceResponse):
    """
    List IP whitelist of instances which allow access to
    """

    def __init__(self, bce_response):
        super(ListSecurityIpResponse, self).__init__()
        self.security_ips = bce_response.security_ips


class ListParameterResponse(BceResponse):
    """
    List configuration parameters and runtime parameters of scs instance
    """

    def __init__(self, bce_response):
        super(ListParameterResponse, self).__init__()
        self.parameters = bce_response.parameters


class Parameter(object):
    """
    Configuration parameters and runtime parameters of scs instance
    """

    def __init__(self, obj):
        super(Parameter, self).__init__()
        self.default = str(obj.default)
        self.force_restart = int(obj.force_restart)
        self.name = str(obj.name)
        self.value = str(obj.value)


class ModifyParameterRequest(object):
    """
    A request model to modify instance parameter
    """

    def __init__(self, obj):
        super(ModifyParameterRequest, self).__init__()
        self.name = str(obj.name)
        self.value = str(obj.value)


class ListBackupResponse(BceResponse):
    """
    List backups of instance.
    """

    def __init__(self, bce_response):
        super(ListBackupResponse, self).__init__()
        self.total_count = bce_response.total_count
        self.backups = bce_response.backups


class Backup(object):
    """
    Backup model
    """

    def __init__(self, obj):
        super(Backup, self).__init__()
        self.backup_type = str(obj.backup_type)
        self.comment = str(obj.comment)
        self.records = obj.records
        self.start_time = str(obj.start_time)


class BackupRecord(object):
    """
    Backup record model
    """

    def __init__(self, obj):
        super(BackupRecord, self).__init__()
        self.backup_record_id = str(obj.backup_record_id)
        self.backup_status = str(obj.backup_status)
        self.duration = int(obj.duration)
        self.object_size = int(obj.object_size)
        self.shard_name = str(obj.shard_name)
        self.start_time = str(obj.start_time)


class GetBackupResponse(BceResponse):
    """
    Get backup detail response model
    """

    def __init__(self, bce_response):
        super(GetBackupResponse, self).__init__()
        self.url = str(bce_response.url)
        self.url_expiration = str(bce_response.url_expiration)


class JsonWrapper(JSONEncoder):
    """
        custom json encoder for class
    """

    def default(self, obj):
        if isinstance(obj, bytes):
            return str(obj, encoding='utf-8')
        if isinstance(obj, SubnetMap):
            return {
                'zoneName': obj.zone_name,
                'subnetId': obj.subnet_id
            }
        if isinstance(obj, Tag):
            return {
                'tagKey': obj.tag_key,
                'tagValue': obj.tag_value
            }
        return JSONEncoder.default(self, obj)



#! usr/bin/python
# -*-coding:utf-8 -*-
# Copyright 2014 Baidu, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions
# and limitations under the License.


"""
This module provides a client class for SCS.
"""
from __future__ import unicode_literals

import copy
import json
import logging
import uuid

import http.client

import baidubce.services.scs.model as model
from baidubce import utils
from baidubce.auth import bce_v1_signer
from baidubce.bce_base_client import BceBaseClient
from baidubce.bce_client_configuration import BceClientConfiguration
from baidubce.exception import BceClientError
from baidubce.exception import BceServerError
from baidubce.http import bce_http_client
from baidubce.http import http_headers
from baidubce.http import http_methods
from baidubce.services import scs
from baidubce.utils import required

_logger = logging.getLogger(__name__)


def _parse_result(http_response, response):
    if http_response.status / 100 == http.client.CONTINUE / 100:
        raise BceClientError('Can not handle 1xx http status code')
    bse = None
    body = http_response.read()
    if body:
        d = json.loads(body)

        if 'message' in d and 'code' in d and 'requestId' in d:
            r_code = d['code']
            # 1000 means success
            if r_code != '1000':
                bse = BceServerError(d['message'],
                                     code=d['code'],
                                     request_id=d['requestId'])
            else:
                response.__dict__.update(
                    json.loads(body, object_hook=utils.dict_to_python_object).__dict__)
                http_response.close()
                return True
        elif http_response.status / 100 == http.client.OK / 100:
            response.__dict__.update(
                json.loads(body, object_hook=utils.dict_to_python_object).__dict__)
            http_response.close()
            return True
    elif http_response.status / 100 == http.client.OK / 100:
        return True

    if bse is None:
        bse = BceServerError(http_response.reason, request_id=response.metadata.bce_request_id)
    bse.status_code = http_response.status
    raise bse


class ScsClient(BceBaseClient):
    """
    Scs sdk client
    """

    def __init__(self, config=None):
        if config is not None:
            self._check_config_type(config)
        BceBaseClient.__init__(self, config)

    @required(config=BceClientConfiguration)
    def _check_config_type(self, config):
        return True

    @required(engine_version=(str), instance_name=(str), cluster_type=(str),
              node_type=(str), shard_num=int, proxy_num=int,
              replication_num=int, store_type=int, vpc_id=(str), subnets=list, port=int, purchase_count=int,
              auto_renew_time_unit=(str), auto_renew_time=int, billing=model.Billing)
    def create_instance(self, engine_version, instance_name, cluster_type, node_type, shard_num, replication_num,
                        proxy_num=0, store_type=None, vpc_id=None, subnets=None, port=6379, purchase_count=1,
                        auto_renew_time_unit='month', auto_renew_time=None, billing=model.Billing(), config=None):
        """
        Create instance with specific config

        :param engine_version:
        :type engine_version:string or unicode

        :param instance_name: Instance name
        :type  name: string or unicode

        :param cluster_type: default/master_slave or cluster
        :type  cluster_type: string or unicode

        :param node_type: Node specification
        :type  node_type: string or unicode

        :param shard_num: number of shard
        :type  shard_num: int

        :param replication_num: number of replication
        :type  replication_num: int

        :param store_type: default is 0;High performance memory:0;ssd-native:1
        :type  store_type: int

        :param vpc_id: vpc id,use default vpc id if not provide.
        :type  vpc_id: str

        :param subnets: list of model.SubnetMap
        :type  subnets: list

        :param port: default is 6379
        :type  port: int

        :param purchase_count: Number of purchases, default is 1
        :type  purchase_count: int

        :param auto_renew_time_unit: Renew monthly or yearly,value in ['month','year]
        :type  auto_renew_time_unit: str

        :param auto_renew_time: If billing is Prepay, the automatic renewal time is 1-9
         when auto_renew_time_unit is 'month' and 1-3 when auto_renew_time_unit is 'year'
        :type  auto_renew_time: int

        :param billing: default billing is Prepay 1 month
        :type  billing: model.Billing

        :param config: None
        :type  config: BceClientConfiguration

        :return: Object
            {
                "instance_ids": ["scs-bj-wHoJXL09355"]
            }

        :rtype: baidubce.bce_response.BceResponse
        """
        data = {
            'billing': billing.__dict__,
            'instanceName': instance_name,
            'nodeType': node_type,
            'port': port,
            'engineVersion': engine_version,
            'storeType': store_type,
            'purchaseCount': purchase_count,
            'shardNum': shard_num,
            'proxyNum': proxy_num,
            'replicationNum': replication_num,
            'clusterType': cluster_type,
            'vpcId': vpc_id,
            'subnets': subnets,
            'autoRenewTimeUnit': auto_renew_time_unit,
            'autoRenewTime': auto_renew_time
        }

        return model.CreateInstanceResponse(self._send_request(http_methods.POST, 'instance',
                                                               params={"clientToken": uuid.uuid4()},
                                                               body=json.dumps(data, cls=model.JsonWrapper, indent=4),
                                                               config=config,
                                                               api_version=2))

    @required(engine_version=(str), instance_id=(str),
              node_type=(str), shard_num=int, proxy_num=int, billing=model.Billing)
    def resize_instance(self, instance_id, node_type, shard_num=None, engine_version=None, billing=None, config=None):
        """
        Create instance with specific config

        :param engine_version:
        :type engine_version:string or unicode

        :type  name: string or unicode

        :param node_type: Node specification
        :type  node_type: string or unicode

        :param instance_id: number of shard
        :type  shard_num: int

        :param shard_num: number of proxy node
        :type  shard_num: int

        :param billing: default billing is Prepay 1 month
        :type  billing: model.Billing

        :param config: None
        :type  config: BceClientConfiguration

        """
        response = self.get_instance_detail(instance_id)
        billing = model.Billing(pay_method=response.payment_timing)
        if engine_version is None:
            engine_version = response.engine_version
        if shard_num is None:
            shard_num = response.shard_num
        data = {
            'billing': billing.__dict__,
            'nodeType': node_type,
            'engineVersion': engine_version,
            'shardNum': shard_num
        }
        key = instance_id + '/resize'
        return model.CreateInstanceResponse(self._send_request(http_methods.PUT, 'instance', key=key,
                                                               body=json.dumps(data), config=config))

    @required(instance_id=(str))
    def delete_instance(self, instance_id, config=None):
        """
        Delete instance
        :param instance_id: The ID of instance
        :type  instance_id: string or unicode

        :param config: None
        :type  config: BceClientConfiguration

        :return:
        """
        return self._send_request(http_method=http_methods.DELETE, function_name='instance', key=instance_id,
                                  config=config, api_version=1)

    def list_instances(self, marker=None, max_keys=1000, config=None):
        """
        Get instances in current region
        :param marker: start position
        :param max_keys: max count per page
        :param config:
        :return:
        """
        params = {}
        if marker is not None:
            params[b'marker'] = marker
        if max_keys is not None:
            params[b'maxKeys'] = max_keys
        return model.ListInstanceResponse(self._send_request(http_methods.GET,
                                                             function_name='instance',
                                                             params=params,
                                                             config=config))

    @required(instance_id=(str))
    def get_instance_detail(self, instance_id, config=None):
        """
        Get instance detail
        :param instance_id: The ID of instance
        :type  instance_id: string or unicode

        :param config: None
        :type  config: BceClientConfiguration

        :return:
            {
                "instance_id":"scs-bj-cxisuftlkquj",
                "instance_name":"post101",
                "instance_status":"_running",
                "cluster_type":"master_slave",
                "engine":"redis",
                "engine_version":"3.2",
                "vnet_ip":"10.107.231.11",
                "domain":"redis.zsfqaybijgbv.scs.bj.baidubce.com",
                "port":"6379",
                "instance_create_time":"2018-11-13_t05:37:49_z",
                "capacity":1,
                "used_capacity":0.06,
                "payment_timing":"_postpaid",
                "vpc_id":"vpc-1n1wqxfu4iuu",
                "auto_renew":"true",
                "subnets":[
                    {
                        "name":"系统预定义子网_c",
                        "subnet_id":"sbn-0ynnfkyymh8z",
                        "zone_name":"cn-bj-c",
                        "cidr":"192.168.32.0/20"
                    },
                    {
                        "name":"系统预定义子网",
                        "subnet_id":"sbn-rvv87cdd0gv9",
                        "zone_name":"cn-bj-a",
                        "cidr":"192.168.0.0/20"
                    }
                ],
                "zone_names":[
                    "cn-bj-a",
                    "cn-bj-c"
                ]
            }
        """
        return model.GetInstanceResponse(self._send_request(http_methods.GET, function_name='instance', key=instance_id,
                                                            config=config, api_version=2))

    @required(instance_id=(str), instance_name=(str))
    def rename_instance(self, instance_id, instance_name, config=None):
        """
        Update the instance_name of instance
        :param instance_id: the ID of instance
        :type str

        :param instance_name: a new instance name for instance
        :type str

        :param config: None
        :type BceClientConfiguration

        :return:
        """
        data = {'instanceName': instance_name}
        key = instance_id + '/rename'
        return self._send_request(http_method=http_methods.PUT, function_name='instance', key=key,
                                  body=json.dumps(data), config=config, api_version=1)

    @required(instance_id=(str))
    def restart_instance(self, instance_id, config=None):
        """
        restart the instance
        :param instance_id: the ID of instance
        :type str

        :type str

        :param config: None
        :type BceClientConfiguration

        :return:
        """
        key = instance_id + '/restart'
        return self._send_request(http_method=http_methods.PUT, function_name='instance', key=key,
                                  config=config, api_version=2)

    @required(instance_id=(str), domain=(str))
    def rename_instance_domain(self, instance_id, domain, config=None):
        """
        Update the domain of instance
        :param instance_id: the ID of instance
        :type str

        :param domain: a new domain for instance.
        New instance domain name.
        Naming rules of domain name:
            1. It is composed of lowercase letters and numbers;
            2. It begins with a lowercase letter;
            3. The length is between 3-30
        :type str

        :param config: None
        :type BceClientConfiguration

        :return:
        """
        data = {'domain': domain}
        key = instance_id + '/renameDomain'
        return self._send_request(http_method=http_methods.PUT, function_name='instance', key=key,
                                  body=json.dumps(data), config=config, api_version=1)

    @required(instance_id=(str), password=(str))
    def flush_instance(self, instance_id, password='', config=None):
        """
        Clear the data of SCS instance
        :param instance_id: the ID of instance
        :type str

        :param password: the password of instance.
        if no password is set, an empty string is passed.
        The password needs to be encrypted.
        Please refer to the definition of password encryption transmission specification for details.
        :type str

        :param config: None
        :type BceClientConfiguration

        :return:
        """
        password = ''
        if len(password) > 0:
            secret_access_key = self.config.credentials.secret_access_key
            password = utils.aes128_encrypt_16char_key(password, secret_access_key)
        data = {
            'password': password
        }
        key = instance_id + '/flush'
        return self._send_request(http_method=http_methods.PUT, function_name='instance', key=key,
                                  body=json.dumps(data), config=config, api_version=1)

    def list_available_zones(self, config=None):
        """
        lost available zone of current region
        :param config:
        :type BceClientConfiguration
        :return:
        """
        return model.ListAvailableZoneResponse(self._send_request(http_methods.GET, function_name="zone",
                                                                  config=config))

    def list_subnets(self, vpc_id=None, zone_name=None, config=None):
        """
        list subnet by vpcId or zoneName
        :param vpc_id: ID of the VPC to which it belongs
        :param zone_name: The name of the zone it belongs to
        :param config:
        :type BceClientConfiguration
        :return:
        """
        params = {}
        if vpc_id is not None:
            params[b'vpcId'] = vpc_id
        if zone_name is not None:
            params[b'zoneName'] = zone_name
        return model.ListSubnetResponse(self._send_request(http_methods.GET, function_name="subnet",
                                                           params=params, config=config))

    def list_nodetypes(self, config=None):
        """
        List nodetypes
        :param config:
        :type BceClientConfiguration
        :return:
        """
        return model.ListNodeTypeResponse(self._send_request(http_methods.GET, function_name="nodetypes",
                                                             config=config, api_version=2))

    @required(instance_id=(str), change_tags=list)
    def bind_tags(self, instance_id, change_tags, config=None):
        """
        Bind tags to instance
        :param change_tags: tag list to bind to instance
        :type list of Tag

        :param instance_id: the ID of instance
        :type str

        :param config: None
        :type BceClientConfiguration

        :return:
        """

        data = {
            'changeTags': change_tags
        }
        key = instance_id + '/bindTag'
        return self._send_request(http_method=http_methods.PUT, function_name='instance',
                                  key=key, body=json.dumps(data, cls=model.JsonWrapper, indent=4), config=config)

    @required(instance_id=(str), change_tags=list)
    def unbind_tags(self, instance_id, change_tags, config=None):
        """
        Unbind tags to instance
        :param change_tags: tag list to unbind
        :type list of Tag

        :param instance_id: the ID of instance
        :type str

        :param config: None
        :type BceClientConfiguration

        :return:
        """
        data = {
            'changeTags': change_tags
        }
        key = instance_id + '/unBindTag'
        return self._send_request(http_method=http_methods.PUT, function_name='instance',
                                  key=key, body=json.dumps(data, cls=model.JsonWrapper, indent=4), config=config)

    @required(instance_id=(str))
    def set_as_master(self, instance_id, config=None):
        """
        Set an SCS instance as the primary region of the live instance group
        The cluster created by default is master
        :param instance_id: the ID of instance
        :type str

        :param config: None
        :type BceClientConfiguration

        :return:
        """

        key = instance_id + '/setAsMaster'
        return self._send_request(http_method=http_methods.PUT, function_name='instance',
                                  key=key, config=config)

    @required(instance_id=(str), master_domain=(str), master_port=int)
    def set_as_slave(self, instance_id, master_domain, master_port, config=None):
        """
        Set SCS instance A as a slave of a master SCS instance B
        :param master_port: the port of master redis B
        :type int

        :param master_domain:the domain of master redis B
        if B in a different region,peer-2-peer dns copy muster be turn on.
        :type str/unicode

        :param instance_id: the ID of instance
        :type str/unicode

        :param config: None
        :type BceClientConfiguration

        :return:
        """
        data = {
            'masterDomain': master_domain,
            'masterPort': master_port
        }
        key = instance_id + '/setAsSlave'
        return self._send_request(http_method=http_methods.PUT, function_name='instance',
                                  key=key, body=json.dumps(data), config=config)

    @required(instance_id=(str))
    def list_security_ip(self, instance_id, config=None):
        """
        List securityIps of a instance
        :param instance_id: ID of instanceId
        :param config:
        :return:
        """
        key = instance_id + "/securityIp"
        return model.ListSecurityIpResponse(self._send_request(http_methods.GET, function_name="instance",
                                                               key=key, config=config))

    @required(instance_id=(str), security_ips=list)
    def add_security_ips(self, instance_id, security_ips, config=None):
        """
        Add security_ips to access to instance
        :param security_ips:
        :type list

        :param instance_id: the ID of instance
        :type str

        :param config: None
        :type BceClientConfiguration

        :return:
        """
        data = {
            'securityIps': security_ips
        }
        key = instance_id + '/securityIp'
        return self._send_request(http_method=http_methods.PUT, function_name='instance',
                                  key=key, body=json.dumps(data), config=config)

    @required(instance_id=(str), security_ips=list)
    def delete_security_ips(self, instance_id, security_ips, config=None):
        """
        Delete security_ips to access to instance
        :param security_ips:
        :type list

        :param instance_id: the ID of instance
        :type str

        :param config: None
        :type BceClientConfiguration

        :return:
        """
        data = {
            'securityIps': security_ips
        }
        key = instance_id + '/securityIp'
        return self._send_request(http_method=http_methods.DELETE, function_name='instance',
                                  key=key, body=json.dumps(data), config=config)

    @required(instance_id=(str), password=(str))
    def modify_password(self, instance_id, password, config=None):
        """
        Update the password for scs instance
        :param password:
        :type list

        :param instance_id: the ID of instance
        :type str

        :param config: None
        :type BceClientConfiguration

        :return:
        """

        secret_access_key = self.config.credentials.secret_access_key
        data = {
            'password': utils.aes128_encrypt_16char_key(password, secret_access_key)
        }
        key = instance_id + '/modifyPassword'
        return self._send_request(http_method=http_methods.PUT, function_name='instance',
                                  key=key, body=json.dumps(data), config=config)

    @required(instance_id=(str))
    def list_parameters(self, instance_id, config=None):
        """
        List parameters of a instance
        :param instance_id: ID of instanceId
        :param config:
        :return:
        """
        key = instance_id + "/parameter"
        return model.ListParameterResponse(self._send_request(http_methods.GET, function_name="instance",
                                                              key=key, config=config))

    @required(instance_id=(str), name=(str), time=(str), expire_days=int)
    def modify_parameter(self, instance_id, name, value, config=None):
        """
        Update the parameter of instance
        :param name: parameter name
        :param value: parameter value
        :param instance_id: the ID of instance
        :type str

        :param config: None
        :type BceClientConfiguration

        :return:
        """
        data = {
            'parameter': {
                'name': name,
                'value': value
            }
        }
        key = instance_id + '/parameter'
        return self._send_request(http_method=http_methods.PUT, function_name='instance',
                                  key=key, body=json.dumps(data), config=config)

    @required(instance_id=(str))
    def list_backups(self, instance_id, config=None):
        """
        List backup of a instance
        :param instance_id: ID of instanceId
        :param config:
        :return:
        """
        key = instance_id + "/backup"
        return model.ListBackupResponse(self._send_request(http_methods.GET, function_name="instance",
                                                           key=key, config=config))

    @required(instance_id=(str), backup_record_id=(str))
    def get_backup(self, instance_id, backup_record_id, config=None):
        """
        List backup of a instance
        :param backup_record_id: backup record ID
        :param instance_id: ID of instanceId
        :param config:
        :return:
        """
        key = instance_id + "/backup/" + backup_record_id
        return model.GetBackupResponse(self._send_request(http_methods.GET, function_name="instance",
                                                          key=key, config=config))

    @required(instance_id=(str), days=(str), time=(str), expire_days=int)
    def modify_backup_policy(self, instance_id, days, time, expire_days, config=None):
        """
        Update the instance_name of instance
        :param instance_id: the ID of instance
        :type str

        :param expire_days: Backup file expiration time
        :type int

        :param time: when to backup.utc time.eg:01:05:00
        :param days: the duration to backup.eg:Sun,Wed,Thu,Fri,Sta
        :type str

        :param config: None
        :type BceClientConfiguration

        :return:
        """
        data = {
            'backupDays': days,
            'backupTime': time,
            'expireDay': expire_days
        }
        key = instance_id + '/modifyBackupPolicy'
        return self._send_request(http_method=http_methods.PUT, function_name='instance', key=key,
                                  body=json.dumps(data), config=config)

    @staticmethod
    def _get_path_v1(config, function_name=None, key=None):
        return utils.append_uri(scs.URL_PREFIX_V1, function_name, key)

    @staticmethod
    def _get_path_v2(config, function_name=None, key=None):
        return utils.append_uri(scs.URL_PREFIX_V2, function_name, key)

    @staticmethod
    def _bce_scs_sign(credentials, http_method, path, headers, params,
                      timestamp=0, expiration_in_seconds=1800,
                      headers_to_sign=None):

        headers_to_sign_list = [b"host",
                                b"content-md5",
                                b"content-length",
                                b"content-type"]

        if headers_to_sign is None or len(headers_to_sign) == 0:
            headers_to_sign = []
            for k in headers:
                k_lower = k.strip().lower()
                if k_lower.startswith(http_headers.BCE_PREFIX) or k_lower in headers_to_sign_list:
                    headers_to_sign.append(k_lower)
            headers_to_sign.sort()
        else:
            for k in headers:
                k_lower = k.strip().lower()
                if k_lower.startswith(http_headers.BCE_PREFIX):
                    headers_to_sign.append(k_lower)
            headers_to_sign.sort()

        return bce_v1_signer.sign(credentials,
                                  http_method,
                                  path,
                                  headers,
                                  params,
                                  timestamp,
                                  expiration_in_seconds,
                                  headers_to_sign)

    def _merge_config(self, config):
        if config is None:
            return self.config
        else:
            self._check_config_type(config)
            new_config = copy.copy(self.config)
            new_config.merge_non_none_values(config)
            return new_config

    def _send_request(self, http_method, function_name=None, key=None, body=None, headers=None, params=None,
                      config=None, body_parser=None, api_version=1):
        if params is None:
            params = {"clientToken": uuid.uuid4()}
        config = self._merge_config(config)
        path = {
            1: ScsClient._get_path_v1,
            2: ScsClient._get_path_v2,
        }[api_version](config, function_name, key)

        if body_parser is None:
            body_parser = _parse_result

        if headers is None:
            headers = {b'Accept': b'*/*', b'Content-Type': b'application/json;charset=utf-8'}

        return bce_http_client.send_request(config, ScsClient._bce_scs_sign, [body_parser], http_method, path, body,
                                            headers, params)



# Copyright 2014 Baidu, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

"""
This module provides a client class for BMR.
"""

import copy
import logging
import json
import sys


from baidubce.auth import bce_v1_signer
from baidubce import bce_base_client
from baidubce import compat
from baidubce.http import bce_http_client
from baidubce.http import handler
from baidubce.http import http_methods
from baidubce.utils import required
from baidubce.utils import aes128_encrypt_16char_key


_logger = logging.getLogger(__name__)

if sys.version_info[0] == 2:
    value_type = (str, unicode)
else:
    value_type = (str, bytes)


class BmrClient(bce_base_client.BceBaseClient):
    """
    Bmr sdk client
    """

    prefix = '/v1'

    def __init__(self, config=None):
        bce_base_client.BceBaseClient.__init__(self, config)

    @required(image_type=value_type,
              image_version=value_type,
              instance_groups=list)
    def create_cluster(self,
                       image_type,
                       image_version,
                       instance_groups,
                       client_token=None,
                       applications=None,
                       auto_terminate=None,
                       log_uri=None,
                       name=None,
                       steps=None,
                       service_ha_enabled=None,
                       safe_mode_enabled=None,
                       admin_pass=None,
                       vpc_id=None,
                       subnet_id=None,
                       security_group=None,
                       availability_zone=None,
                       templateType=None):
        """
        Create cluster

        :param image_type: the type of virtual machine image
        :type image_type: string

        :param image_version: the version of virtual machine image
        :type image_version: string

        :param instance_groups: instance groups for cluster
        :type instance_groups: array

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = '/cluster'
        params = None
        if client_token is not None:
            params = {
                'clientToken': client_token
            }
        body = {
            'imageType': compat.convert_to_string(image_type),
            'imageVersion': compat.convert_to_string(image_version),
            'instanceGroups': instance_groups
        }
        if applications is not None:
            body['applications'] = applications
        if auto_terminate is not None:
            body['autoTerminate'] = auto_terminate
        if name is not None:
            body['name'] = name
        if log_uri is not None:
            body['logUri'] = log_uri
        if steps is not None:
            body['steps'] = steps
        if service_ha_enabled is not None:
            body['serviceHaEnabled'] = service_ha_enabled
        if safe_mode_enabled is not None:
            body['safeModeEnabled'] = safe_mode_enabled
        if admin_pass is not None and self.config is not None:
            secret_access_key = self.config.credentials.secret_access_key
            body['adminPassword'] = aes128_encrypt_16char_key(admin_pass, secret_access_key)
        if vpc_id is not None:
            body['vpcId'] = vpc_id
        if subnet_id is not None:
            body['subnetId'] = subnet_id
        if security_group is not None:
            body['securityGroup'] = security_group
        if availability_zone is not None:
            body['availabilityZone'] = availability_zone
        if templateType is not None:
            body['templateType'] = templateType

        return self._send_request(http_methods.POST, path, params=params, body=json.dumps(body))

    def list_clusters(self, marker=None, max_keys=None):
        """
        List clusters

        :param marker:
        :type marker: string

        :param max_keys: max records returned.
        :type max_keys: int

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = '/cluster'
        params = None
        if marker is not None or max_keys is not None:
            params = {}
        if marker is not None:
            params['marker'] = marker
        if max_keys is not None:
            params['maxKeys'] = max_keys

        return self._send_request(http_methods.GET, path, params=params)

    @required(cluster_id=value_type)
    def get_cluster(self, cluster_id):
        """
        Get cluster

        :param cluster_id: cluster id
        :type cluster_id: string

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = '/cluster/%s' % compat.convert_to_string(cluster_id)
        return self._send_request(http_methods.GET, path)
    @required(cluster_id=value_type)
    def list_cluster_hosts(self, cluster_id):
        """
        Get cluster hosts
        :param cluster_id: cluster id
        :type cluster_id: string

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = '/cluster/%s/hosts' % compat.convert_to_string(cluster_id)
        return self._send_request(http_methods.GET, path)

    @required(cluster_id=value_type)
    def get_cluster_ambariPassword(self, cluster_id):
        """
        Get cluster ambari password

        :param cluster_id: cluster id
        :type cluster_id: string

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = '/cluster/%s/ambaripassword' % compat.convert_to_string(cluster_id)
        return self._send_request(http_methods.GET, path)

    @required(cluster_id=value_type)
    def terminate_cluster(self, cluster_id):
        """
        Terminate cluster

        :param cluster_id: cluster id
        :type cluster_id: string

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = '/cluster/%s' % compat.convert_to_string(cluster_id)
        return self._send_request(http_methods.DELETE, path)



    @required(cluster_id=value_type,
              instance_group_config=list)
    def scale_cluster(self, cluster_id, instance_group_config):
        """
        Scale cluster
        :param cluster_id: cluster id
        :type cluster_id: string

        :param instance_group_id: instance group id
        :type instance_group_id: string

        :param instance_count: instance count
        :type instance_count: int

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = "/cluster/%s/instanceGroup" % compat.convert_to_string(cluster_id)
        params = None
        body = {
            "instanceGroups": instance_group_config
        }
        return self._send_request(http_methods.PUT, path, params=params, body=json.dumps(body))


    @required(cluster_id=value_type, steps=list)
    def add_steps(self, cluster_id, steps, client_token=None):
        """
        Add steps

        :param cluster_id: cluster id
        :type cluster_id: string

        :param steps: steps to be added
        :type steps: Array

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = '/cluster/%s/step' % compat.convert_to_string(cluster_id)
        params = None
        if client_token is not None:
            params = {
                'clientToken': client_token
            }
        body = json.dumps({'steps': steps})
        return self._send_request(http_methods.POST, path, params=params, body=body)

    @required(cluster_id=value_type)
    def list_steps(self, cluster_id, pageNo=None, pageSize=None):
        """
        List step

        :param cluster_id: cluster id
        :type cluster_id: string

        :param marker:
        :type marker: string

        :param max_keys: max records returned.
        :type max_keys: int

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = '/cluster/%s/step' % compat.convert_to_string(cluster_id)
        params = None
        if pageNo is not None or pageSize is not None:
            params = {}
        if pageNo is not None:
            params['pageNo'] = pageNo
        if pageSize is not None:
            params['pageSize'] = pageSize

        return self._send_request(http_methods.GET, path, params=params)

    @required(cluster_id=value_type, step_id=value_type)
    def get_step(self, cluster_id, step_id):
        """
        Get step

        :param cluster_id: cluster id
        :type cluster_id: string

        :param step_id: step id
        :type step_id: string

        :return: baidubce.bce_response.BceResponse
        :rtype baidubce.bce_response.BceResponse
        """
        path = '/cluster/%s/step/%s' % (compat.convert_to_string(cluster_id),
                compat.convert_to_string(step_id))
        return self._send_request(http_methods.GET, path)
    @required(
        clusterId=value_type,
        stepId=value_type
    )
    def cancel_step(self, clusterId, stepId):
        """

        :param clusterId:cluster id
        :param stepId: step id
        :return: baidubce.bce_response.BceResponse
        """
        path = '/cluster/%s/step/%s' % (clusterId, stepId)
        return self._send_request(http_methods.DELETE, path)

    @required(cluster_id=value_type, instance_group_id=value_type)
    def list_instances(self, cluster_id, instance_group_id):
        """
        List instances

        :param cluster_id: cluster id
        :type cluster_id: string

        :param instance_group_id: instance group id
        :type instance_group_id: string

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = '/cluster/%s/instanceGroup/%s/instance' % (compat.convert_to_string(cluster_id),
                compat.convert_to_string(instance_group_id))
        return self._send_request(http_methods.GET, path)

    @required(cluster_id=value_type)
    def list_instance_groups(self, cluster_id):
        """
        List instance groups

        :param cluster_id: cluster id
        :type cluster_id: string

        :return:
        :rtype: baidubce.bce_response.BceResponse
        """
        path = '/cluster/%s/instanceGroup' % compat.convert_to_string(cluster_id)
        return self._send_request(http_methods.GET, path)


    @required(cluster_id=value_type)
    def describe_cluster(self, cluster_id):
        """
        :param cluster_id: clusterId
        :type  cluster_id: string
        :return: baidubce.bce_response.BceResponse
        """
        params = None
        body = {
            'clusterId': cluster_id
        }
        path ='/cluster/detail'
        return self._send_request(http_methods.POST, path, params=params, body=json.dumps(body))

    @required(clusterId=value_type,
              newName=value_type)
    def rename_cluster(self, clusterId, newName):
        """
        rename a cluster
        :param clusterId: clusterId
        :type  clusterId: string
        :param newName: newName
        :type  newName: string
        :return: baidubce.bce_response.BceResponse
        """
        if newName is None or len(newName.strip(" ")) == 0:
            raise ValueError("newName can not be empty.")
        params = None
        body = {
            'clusterId': clusterId,
            'newName': newName
        }
        path = '/cluster/rename'
        return self._send_request(http_methods.POST, path, params=params, body=json.dumps(body))


    def _merge_config(self, config=None):
        if config is None:
            return self.config
        else:
            new_config = copy.copy(self.config)
            new_config.merge_non_none_values(config)
            return new_config

    def _send_request(self, http_method, path,
                      body=None, headers=None, params=None,
                      config=None, body_parser=None):
        config = self._merge_config(config)
        if body_parser is None:
            body_parser = handler.parse_json

        config.endpoint = compat.convert_to_bytes(config.endpoint)
        return bce_http_client.send_request(
            config, sign_wrapper([b'host', b'x-bce-date']), [handler.parse_error, body_parser],
            http_method, compat.convert_to_bytes(BmrClient.prefix + path), body, headers, params)


def sign_wrapper(headers_to_sign):
    """wrapper the bce_v1_signer.sign()."""
    def _wrapper(credentials, http_method, path, headers, params):
        credentials.access_key_id = compat.convert_to_bytes(credentials.access_key_id)
        credentials.secret_access_key = compat.convert_to_bytes(credentials.secret_access_key)

        return bce_v1_signer.sign(credentials,
                                  compat.convert_to_bytes(http_method),
                                  compat.convert_to_bytes(path), headers, params,
                                  headers_to_sign=headers_to_sign)
    return _wrapper


def instance_group(group_type, instance_type, instance_count,
                   name=None, cds=None, root_DiskSizeInGB=40, root_DiskMediumType='ssd'):
    """
    Construct instance group

    :param group_type: instance group type
    :type group_type: ENUM {'Master', 'Core', 'Task'}

    :param instance_type
    :type instance_type: ENUM {'g.small', 'c.large', 'm.medium', 's.medium', 'c.2xlarge'}

    :param instance_count
    :type instance_count: int

    :param name: instance group name
    :type name: string
    """
    instance_group = {
        'type': group_type,
        'instanceCount': instance_count,
        'instanceType': instance_type,
        'rootDiskSizeInGB': root_DiskSizeInGB,
        'rootDiskMediumType': root_DiskMediumType,
        'cds': cds
    }
    if name is not None:
        instance_group['name'] = name

    return instance_group


def cdsitem(sizeInGB, mediumType):
    """
    :param sizeInGB: sizeInGB
    :type string
    :param mediumType: mediumType
    :type string ssd, sata, premium_ssd
    :return: cds
    """
    cds = {
        'sizeInGB': sizeInGB,
        'mediumType': mediumType
     }
    return cds
def application(name, version, properties=None):
    """
    Construct application

    :param name: application type
    :type name: ENUM {'hadoop', 'spark', 'hive', 'pig', 'hbase', 'hue', 'zeppelin', 'kafka', 'mahout'}

    :param version: application version
    :type version: string

    :param properties: application properties
    :type properties: dict
    """
    application = {
        'name': name,
        'version': version
    }
    if properties is not None:
        application['properties'] = properties
    return application


def step(step_type, action_on_failure, properties, name=None, additional_files=None):
    """
    Create step

    :param step_type: the type of step
    :type step_type: Enum {'Java','Streaming','Hive','Pig', 'Spark'}

    :param action_on_failure
    :type actionOnFailure: Enum {'Continue','TerminateCluster','CancelAndWait'}

    :param properties: step properties
    :type properties: dict

    :param name: the name of the step
    :type name: string

    :param additional_files: list of step additional file
    :type additional_files: list
    """
    step = {
        'actionOnFailure': action_on_failure,
        'type': step_type,
        'properties': properties
    }
    if name is not None:
        step['name'] = name
    if additional_files is not None and len(additional_files) > 0:
        step['additionalFiles'] = additional_files
    return step


def java_step_properties(jar, main_class, arguments=None):
    """
    Create java step properties

    :param jar: the path of .jar file
    :type jar: string

    :param main_class: the package path for main class
    :type main_class: string

    :param arguments: arguments for the step
    :type arguments: string

    :return:
    :rtype map
    """
    java_step = {
        'jar': jar,
        'mainClass': main_class
    }
    if arguments is not None:
        java_step['arguments'] = arguments
    return java_step


def streaming_step_properties(input, output, mapper, reducer=None, arguments=None):
    """
    Create streaming step properties

    :param input: the input path of step
    :type input: string

    :param output: the output path of step
    :type output: string

    :param mapper: the mapper program of step
    :type mapper: string

    :param reducer: the reducer program of step
    :type reducer: string

    :param arguments: arguments for the step
    :type arguments: string

    :return:
    :rtype map
    """
    streaming_step = {
        'mapper': mapper,
        'reducer': '',
        'input': input,
        'output': output
    }
    if reducer is not None:
        streaming_step['reducer'] = reducer
    if arguments is not None:
        streaming_step['arguments'] = arguments
    return streaming_step


def pig_step_properties(script, arguments=None, input=None, output=None):
    """
    Create pig step properties

    :param script: the script path of step
    :type script: string

    :param arguments: arguments for the step
    :type arguments: string

    :param input: the input path of step
    :type input: string

    :param output: the output path of step
    :type output: string

    :return:
    :rtype map
    """
    pig_step = {
        'script': script
    }
    if arguments is not None:
        pig_step['arguments'] = arguments
    if input is not None:
        pig_step['input'] = input
    if output is not None:
        pig_step['output'] = output
    return pig_step


def hive_step_properties(script, arguments=None, input=None, output=None):
    """
    Create hive step properties

    :param script: the script path of step
    :type script: string

    :param arguments: arguments for the step
    :type arguments: string

    :param input: the input path of step
    :type input: string

    :param output: the output path of step
    :type output: string

    :return:
    :rtype map
    """
    hive_step = {
        'script': script
    }
    if arguments is not None:
        hive_step['arguments'] = arguments
    if input is not None:
        hive_step['input'] = input
    if output is not None:
        hive_step['output'] = output
    return hive_step


def spark_step_properties(jar, submitOptions, arguments=None):
    """
    Create spark step properties

    :param jar: the path of .jar file
    :type jar: string

    :param main_class: the package path for main class
    :type main_class: string

    :param arguments: arguments for the step
    :type arguments: string

    :return:
    :rtype map
    """
    spark_step = {
        'jar': jar,
        'submitOptions': submitOptions
    }
    if arguments is not None:
        spark_step['arguments'] = arguments
    return spark_step


def additional_file(remote, local):
    """
    Create step additional file

    :param remote: the remote file of the additional file
    :type remote: string

    :param local: the local file of the additional file
    :type local: string

    :return:
    :rtype map
    """
    return {
        'remote': remote,
        'local': local,
    }
def instancegroup_config(id, instanceCount):
    """
    create instance_group
    :param id: id
    :type id: string
    :param instanceCount: instanceCount
    :type instanceCount: int
    :return: instancegroup_config
    """
    instancegroup_config = {
        'id': id,
        'instanceCount': instanceCount
    }
    return instancegroup_config
@required(
    period=int,
    periodUnit=value_type,
)
def schedule_properties(period, periodUnit, startTime=None, endTime=None):
    """
    :param period: period
    :type id: int
    :param periodUnit: periodUnit
    :type id: string
    :param startTime: startTime
    :param endTime: endTime
    :return: schedule
    """
    schedule = {
        'period': period,
        'periodUnit': periodUnit,
    }
    if startTime is not None:
        schedule['startTime'] = startTime
    if endTime is not None:
        schedule['endTime'] = endTime
    return schedule


# Copyright 2014 Baidu, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

"""
This module defines a common configuration class for BCE.
"""

from future.utils import iteritems
from builtins import str
from builtins import bytes
import baidubce.protocol
import baidubce.region
from baidubce.retry.retry_policy import BackOffRetryPolicy
from baidubce import compat


class BceClientConfiguration(object):
    """Configuration of Bce client."""

    def __init__(self,
                 credentials=None,
                 endpoint=None,
                 protocol=None,
                 region=None,
                 connection_timeout_in_mills=None,
                 send_buf_size=None,
                 recv_buf_size=None,
                 retry_policy=None,
                 security_token=None,
                 cname_enabled=False,
                 backup_endpoint=None):
        self.credentials = credentials
        self.endpoint = compat.convert_to_bytes(endpoint) if endpoint is not None else endpoint
        self.protocol = protocol
        self.region = region
        self.connection_timeout_in_mills = connection_timeout_in_mills
        self.send_buf_size = send_buf_size
        self.recv_buf_size = recv_buf_size
        if retry_policy is None:
            self.retry_policy = BackOffRetryPolicy()
        else:
            self.retry_policy = retry_policy
        self.security_token = security_token
        self.cname_enabled = cname_enabled
        self.backup_endpoint = compat.convert_to_bytes(backup_endpoint) if backup_endpoint is not None else backup_endpoint

    def merge_non_none_values(self, other):
        """

        :param other:
        :return:
        """
        for k, v in iteritems(other.__dict__):
            if v is not None:
                self.__dict__[k] = v


DEFAULT_PROTOCOL = baidubce.protocol.HTTP
DEFAULT_REGION = baidubce.region.BEIJING
DEFAULT_CONNECTION_TIMEOUT_IN_MILLIS = 50 * 1000
DEFAULT_SEND_BUF_SIZE = 1024 * 1024
DEFAULT_RECV_BUF_SIZE = 10 * 1024 * 1024
DEFAULT_CONFIG = BceClientConfiguration(
    protocol=DEFAULT_PROTOCOL,
    region=DEFAULT_REGION,
    connection_timeout_in_mills=DEFAULT_CONNECTION_TIMEOUT_IN_MILLIS,
    send_buf_size=DEFAULT_SEND_BUF_SIZE,
    recv_buf_size=DEFAULT_RECV_BUF_SIZE,
    retry_policy=BackOffRetryPolicy())



# -*- coding: utf-8 -*-

# Copyright (c) 2014 Baidu.com, Inc. All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy
#  of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions
# and limitations under the License.

"""
This module provides a client class for EIP.
"""

import copy
import json
import logging
import uuid

from baidubce import utils
from baidubce.auth import bce_v1_signer
from baidubce.bce_base_client import BceBaseClient
from baidubce.http import bce_http_client
from baidubce.http import handler
from baidubce.http import http_methods
from baidubce.services.eip.model import EipStatus

_logger = logging.getLogger(__name__)


class EipClient(BceBaseClient):
    """
    eip sdk client
    """
    version = b'/v1'
    prefix = b'/eip'

    def __init__(self, config=None):
        """
        :type config: baidubce.BceClientConfiguration
        :param config:
        """
        BceBaseClient.__init__(self, config)

    def create_eip(self, bandwidth_in_mbps, name=None, billing=None,
                   client_token=None, config=None):
        """
        Create an eip with the specified options.

        :type bandwidth_in_mbps: int
        :param bandwidth_in_mbps: specify the bandwidth in Mbps

        :type name: string
        :param name: name of eip. The optional parameter

        :type billing: Billing
        :param billing: billing information. The optional parameter,
         default paymentTiming is Postpaid

        :type client_token: string
        :param client_token: if the clientToken is not specified by the user,
         a random String generated by default algorithm will be used.

        :type config: baidubce.BceClientConfiguration
        :param config:

        :return: created eip address, for example,{"eip":"x.x.x.x"}
        """
        body = {
            'name': name,
            'bandwidthInMbps': bandwidth_in_mbps
        }
        if billing is None:
            body['billing'] = {
                'paymentTiming': 'Postpaid',
                'billingMethod': 'ByBandwidth'
            }
        else:
            body['billing'] = {
                'paymentTiming': billing.payment_timing,
                'billingMethod': billing.billing_method,
                'reservation': {
                    'reservationLength': billing.reservation_length,
                    'reservationTimeUnit': billing.reservation_time_unit
                }
            }
        path = self._get_path()
        if client_token is None:
            client_token = self._generate_default_client_token()
        params = {
            b'clientToken': client_token
        }

        return self._send_request(http_methods.POST, path,
                                  body=json.dumps(body), params=params,
                                  config=config)

    def resize_eip(self, eip, new_bandwidth_in_mbps, client_token=None,
                   config=None):
        """
        Resizing eip

        :type eip: string
        :param eip: eip address to be resized

        :type new_bandwidth_in_mbps: int
        :param new_bandwidth_in_mbps: specify new bandwidth in Mbps for eip

        :type client_token: string
        :param client_token: if the clientToken is not specified by the user,
         a random String generated by default algorithm will be used.

        :type config: baidubce.BceClientConfiguration
        :param config:

        :return: BceResponse
        """
        body = {
            'newBandwidthInMbps': new_bandwidth_in_mbps
        }
        path = utils.append_uri(self._get_path(), eip)
        if client_token is None:
            client_token = self._generate_default_client_token()
        params = {
            b'resize': b'',
            b'clientToken': client_token
        }
        return self._send_request(http_methods.PUT, path, params=params,
                                  body=json.dumps(body), config=config)

    def purchase_reserved_eip(self, eip, billing=None, client_token=None,
                              config=None):
        """
        PurchaseReserved eip with fixed duration,only Prepaid eip can do this

        :type eip: string
        :param eip: eip address to be renewed

        :type billing: Billing
        :param billing: billing information. The optional parameter,
         default reservationLength is 1

        :type client_token: string
        :param client_token: if the clientToken is not specified by the user,
         a random String generated by default algorithm will be used.

        :type config: baidubce.BceClientConfiguration
        :param config:

        :return: BceResponse
        """
        if billing is None:
            body = {
                'billing': {
                    'reservation': {
                        'reservationLength': 1,
                        'reservationTimeUnit': 'Month'
                    }
                }
            }
        else:
            body = {
                'billing': {
                    'reservation': {
                        'reservationLength': billing.reservation_length,
                        'reservationTimeUnit': billing.reservation_time_unit
                    }
                }
            }
        path = utils.append_uri(self._get_path(), eip)
        if client_token is None:
            client_token = self._generate_default_client_token()
        params = {
            b'purchaseReserved': b'',
            b'clientToken': client_token
        }
        return self._send_request(http_methods.PUT, path, params=params,
                                  body=json.dumps(body), config=config)

    def bind_eip(self, eip, instance_type, instance_id, client_token=None,
                 config=None):
        """
        bind the eip to a specified instanceId and instanceType

        :type eip: string
        :param eip: eip address to be bound

        :type instance_type: string
        :param instance_type: type of instance to be bound(BCC BLB et.)

        :type instance_id: string
        :param instance_id: id of instance to be bound

        :type client_token: string
        :param client_token: if the clientToken is not specified by the user,
         a random String generated by default algorithm will be used.

        :type config: baidubce.BceClientConfiguration
        :param config:

        :return: BceResponse
        """
        body = {
            'instanceType': instance_type,
            'instanceId': instance_id
        }
        path = utils.append_uri(self._get_path(), eip)
        if client_token is None:
            client_token = self._generate_default_client_token()
        params = {
            b'bind': b'',
            b'clientToken': client_token
        }
        return self._send_request(http_methods.PUT, path, params=params,
                                  body=json.dumps(body), config=config)

    def unbind_eip(self, eip, client_token=None, config=None):
        """
        unbind the eip from a specified instance

        :type eip: string
        :param eip: eip address to be unbound

        :type client_token: string
        :param client_token: if the clientToken is not specified by the user,
         a random String generated by default algorithm will be used.

        :type config: baidubce.BceClientConfiguration
        :param config:

        :return: BceResponse
        """
        path = utils.append_uri(self._get_path(), eip)
        if client_token is None:
            client_token = self._generate_default_client_token()
        params = {
            b'unbind': b'',
            b'clientToken': client_token
        }
        return self._send_request(http_methods.PUT, path, params=params,
                                  config=config)

    def release_eip(self, eip, client_token=None, config=None):
        """
        release the eip(delete operation)
        Only the Postpaid instance or Prepaid which is expired can be released.
        if the eip has been bound, must unbind before releasing.

        :type eip: string
        :param eip: eip address to be released

        :type client_token: string
        :param client_token: if the clientToken is not specified by the user,
         a random String generated by default algorithm will be used.

        :type config: baidubce.BceClientConfiguration
        :param config:

        :return: BceResponse
        """
        path = utils.append_uri(self._get_path(), eip)
        if client_token is None:
            client_token = self._generate_default_client_token()
        params = {
            b'clientToken': client_token
        }
        return self._send_request(http_methods.DELETE, path, params=params,
                                  config=config)

    def list_eips(self, eip=None, instance_type=None, instance_id=None, status=None, marker=None, max_keys=1000,
                  config=None):
        """
        get a list of eip owned by the authenticated user and specified
        conditions. we can Also get a single eip function  through this
        interface by eip condition

        :type eip: string
        :param eip: eip address condition

        :type instance_type: string
        :param instance_type: bound instance type condition

        :type instance_id: string
        :param instance_id: bound instance id condition
        if query by the instanceId or instanceType condition, must provides
         both of them at the same time

        :type status: string
        :param status of eip condition
        if query by the status condition, must provides

        :type marker: string
        :param marker: The optional parameter marker specified in the original
         request to specify where in the results to begin listing.

        :type max_keys: int
        :param max_keys: The optional parameter to specifies the max number
         of list result to return. The default value is 1000.

        :type config: baidubce.BceClientConfiguration
        :param config:

        :return: list of eip model, for example:
                {
                    "eipList": [
                        {
                            "name":"eip-xxxxxxx-1",
                            "eip": "x.x.x.x",
                            "status":"binded",
                            "instanceType": "BCC",
                            "instanceId": "i-xxxxxxx",
                            "bandwidthInMbps": 5,
                            "paymentTiming":"Prepaid",
                            "billingMethod":"ByBandwidth",
                            "createTime":"2016-03-08T08:13:09Z",
                            "expireTime":"2016-04-08T08:13:09Z"
                        },
                        {
                            "name":"eip-xxxxxxx-1",
                            "eip": "x.x.x.x",
                            "status":"binded",
                            "instanceType": "BCC",
                            "instanceId": "i-xxxxxxx",
                            "bandwidthInMbps": 1,
                            "paymentTiming":"Postpaid",
                            "billingMethod":"ByTraffic",
                            "createTime":"2016-03-08T08:13:09Z",
                            "expireTime":null
                        },
                    ],
                    "marker":"eip-xxxxxxx-1",
                    "isTruncated": true,
                    "nextMarker": "eip-DCB50387",
                    "maxKeys": 2
                }
        """
        path = self._get_path()
        params = {}
        if eip is not None:
            params[b'eip'] = eip
        if instance_type is not None:
            params[b'instanceType'] = instance_type
        if instance_id is not None:
            params[b'instanceId'] = instance_id
        if status is not None and isinstance(status, EipStatus):
            params[b'status'] = status.value
        if marker is not None:
            params[b'marker'] = marker
        if max_keys is not None:
            params[b'maxKeys'] = max_keys
        return self._send_request(http_methods.GET, path, params=params,
                                  config=config)

    @staticmethod
    def _generate_default_client_token():
        """
        default client token by uuid1
        """
        return uuid.uuid1()

    @staticmethod
    def _get_path(prefix=None):
        """
        :type prefix: string
        :param prefix: path prefix
        """
        if prefix is None:
            prefix = EipClient.prefix
        return utils.append_uri(EipClient.version, prefix)

    def _merge_config(self, config):
        """

        :type config: baidubce.BceClientConfiguration
        :param config:
        :return:
        """
        if config is None:
            return self.config
        else:
            new_config = copy.copy(self.config)
            new_config.merge_non_none_values(config)
            return new_config

    def _send_request(self, http_method, path, body=None, headers=None,
                      params=None, config=None, body_parser=None):
        """

        :param http_method:
        :param path:
        :param body:
        :param headers:
        :param params:

        :type config: baidubce.BceClientConfiguration
        :param config:

        :param body_parser:

        :return: baidubce.BceResponse
        """
        config = self._merge_config(config)
        if body_parser is None:
            body_parser = handler.parse_json

        if headers is None:
            headers = {b'Accept': b'*/*',
                       b'Content-Type': b'application/json;charset=utf-8'}

        return bce_http_client.send_request(config, bce_v1_signer.sign,
                                            [handler.parse_error, body_parser],
                                            http_method, path, body, headers,
                                            params)



# Copyright (c) 2014 Baidu.com, Inc. All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

"""
This module provides a client class for ROUTE.
"""

import copy
import json
import logging
import uuid

from baidubce import bce_base_client
from baidubce.auth import bce_v1_signer
from baidubce.http import bce_http_client
from baidubce.http import handler
from baidubce.http import http_methods

from baidubce.utils import required
from baidubce import compat

_logger = logging.getLogger(__name__)


class RouteClient(bce_base_client.BceBaseClient):
    """
        Route base sdk client
        """
    prefix = b'/v1'

    def __init__(self, config=None):
        bce_base_client.BceBaseClient.__init__(self, config)

    def _merge_config(self, config=None):
        """
        :param config:
        :type config: baidubce.BceClientConfiguration
        :return:
        """
        if config is None:
            return self.config
        else:
            new_config = copy.copy(self.config)
            new_config.merge_non_none_values(config)
            return new_config

    def _send_request(self, http_method, path,
                      body=None, headers=None, params=None,
                      config=None, body_parser=None):
        config = self._merge_config(config)
        if body_parser is None:
            body_parser = handler.parse_json
        if headers is None:
            headers = {b'Accept': b'*/*', b'Content-Type': b'application/json;charset=utf-8'}
        return bce_http_client.send_request(
            config, bce_v1_signer.sign, [handler.parse_error, body_parser],
            http_method, RouteClient.prefix + path, body, headers, params)

    @required(route_table_id=(bytes, str),
              source_address=(bytes, str),
              destination_address=(bytes, str),
              next_hoop_type=(bytes, str),
              description=(bytes, str))
    def create_route(self, route_table_id, source_address, destination_address,
                     next_hop_type, description, next_hop_id=None, client_token=None,
                     config=None):
        """
        Create a route with the specified options.

        :param route_table_id:
            The id of the route table.
        :type route_table_id: string

        :param source_address:
            The source address of the route.
        :type source_address: string

        :param destination_address:
            The destination address of the route
        :type destination_address: string

        :param next_hop_type:
            route type
            the Bcc type is "custom";
            the VPN type is "vpn";
            the NAT type is "nat";
            the local gateway type is "defaultGateway"
        :type next_hop_type: string

        :param description:
            The option param to describe the route table.
        :type description: string

        :param next_hop_id:
            The next hop id
            when the nexthopType is "defaultGateway",this field can be empty
        :type next_hop_id: string

        :param client_token:
            If the clientToken is not specified by the user, a random String
            generated by default algorithm will be used.
        :type client_token: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = b'/route/rule'
        params = {}

        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token

        body = {
            'routeTableId': compat.convert_to_string(route_table_id),
            'sourceAddress': compat.convert_to_string(source_address),
            'destinationAddress': compat.convert_to_string(destination_address),
            'nexthopType': compat.convert_to_string(next_hop_type),
            'description': compat.convert_to_string(description)
        }

        if next_hop_id is not None:
            body['nexthopId'] = compat.convert_to_string(next_hop_id)

        return self._send_request(http_methods.POST, path, body=json.dumps(body), params=params,
                                  config=config)

    @required(vpc_id=(bytes, str), route_table_id=(bytes, str))
    def get_route(self, vpc_id=None, route_table_id=None, config=None):
        """
        Get the detail information of route table for specific route table or/and vpc.

        :param vpc_id:
            the vpc id
            vpcId and routeTableId cannot be empty at the same time
        :type vpc_id: string

        :param route_table_id:
            the id of the route table
            vpcId and routeTableId cannot be empty at the same time
        :type route_table_id: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = b'/route'
        params = {}
        if route_table_id is not None:
            params[b'routeTableId'] = route_table_id
        if vpc_id is not None:
            params[b'vpcId'] = vpc_id

        return self._send_request(http_methods.GET, path, params=params, config=config)

    @required(route_rule_id=(bytes, str))
    def delete_route(self, route_rule_id, client_token=None, config=None):
        """
        Delete the  specific route rule.

        :param route_rule_id:
            The id of the specified route table.
        :type route_rule_id: string

        :param client_token:
            If the clientToken is not specified by the user, a random String
            generated by default algorithm will be used.
        :type route_table_id: string

        :param config:
        :type config: baidubce.BceClientConfiguration

        :return:
        :rtype baidubce.bce_response.BceResponse
        """
        path = b'/route/rule/%s' % compat.convert_to_bytes(route_rule_id)
        params = {}
        if client_token is None:
            params[b'clientToken'] = generate_client_token()
        else:
            params[b'clientToken'] = client_token
        return self._send_request(http_methods.DELETE, path, params=params, config=config)


def generate_client_token_by_uuid():
    """
    The default method to generate the random string for client_token
    if the optional parameter client_token is not specified by the user.

    :return:
    :rtype string
    """
    return str(uuid.uuid4())


generate_client_token = generate_client_token_by_uuid



# Copyright 2014 Baidu, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

"""
This module defines exceptions for BCE.
"""

from baidubce import utils
from builtins import str
from builtins import bytes

class BceError(Exception):
    """Base Error of BCE."""
    def __init__(self, message):
        Exception.__init__(self, message)


class BceClientError(BceError):
    """Error from BCE client."""
    def __init__(self, message):
        BceError.__init__(self, message)


class BceServerError(BceError):
    """Error from BCE servers."""
    REQUEST_EXPIRED = b'RequestExpired'

    """Error threw when connect to server."""
    def __init__(self, message, status_code=None, code=None, request_id=None):
        BceError.__init__(self, message)
        self.status_code = status_code
        self.code = code
        self.request_id = request_id


class BceHttpClientError(BceError):
    """Exception threw after retry"""
    def __init__(self, message, last_error):
        BceError.__init__(self, message)
        self.last_error = last_error



# Copyright 2014 Baidu, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
# an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
# specific language governing permissions and limitations under the License.
"""
This module provides some python2 and python3's compatible functions for BTS.
"""

import sys
import logging

from baidubce.exception import BceClientError
from baidubce.services.bts import PYTHON_VERSION_ERROR

PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3
if PY3:
    import urllib.parse
else:
    import urllib

_logger = logging.getLogger(__name__)


def _encode(s):
    """
    _encode

    :param s: s
    :type s: string

    :return:
    :rtype string
    """
    if PY2:
        return urllib.quote(s)
    if PY3:
        return urllib.parse.quote(s)

    ex = BceClientError(PYTHON_VERSION_ERROR)
    _logger.debug(ex)
    raise ex


def _decode(s):
    """
    _decode

    :param s: s
    :type s: string

    :return:
    :rtype string
    """
    if PY2:
        return urllib.unquote(s)
    if PY3:
        return urllib.parse.unquote(s)

    ex = BceClientError(PYTHON_VERSION_ERROR)
    _logger.debug(ex)
    raise ex




# Copyright 2014 Baidu, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

"""
This module provides general http handler functions for processing http responses from media services.
"""

import json
from baidubce import utils


def dict_to_python_object(d):
    """
    dict to python object without converting camel key to a "pythonic" name

    :param d:
    :return:
    """
    attr = {}
    for k, v in utils.iteritems(d):
        if not isinstance(k, utils.compat.string_types):
            k = utils.compat.convert_to_string(k)
        attr[k] = v
    return utils.Expando(attr)


def parse_json(http_response, response):
    """If the body is not empty, convert it to a python object and set as the value of
    response.body. http_response is always closed if no error occurs.

    :param http_response: the http_response object returned by HTTPConnection.getresponse()
    :type http_response: httplib.HTTPResponse

    :param response: general response object which will be returned to the caller
    :type response: baidubce.BceResponse

    :return: always true
    :rtype bool
    """
    body = http_response.read()
    if body:
        body = utils.compat.convert_to_string(body)
        response.__dict__.update(json.loads(body, object_hook=dict_to_python_object).__dict__)
    http_response.close()
    return True


def parse_secret_key_response(http_response, response):
    """If the body is not empty, set body content as secretKey attribute of response

    :param http_response: the http_response object returned by HTTPConnection.getresponse()
    :type http_response: httplib.HTTPResponse

    :param response: general response object which will be returned to the caller
    :type response: baidubce.BceResponse

    :return: always true
    :rtype bool
    """
    body = http_response.read()
    if body:
        body = utils.compat.convert_to_string(body)
        response.encryption_key = body
    http_response.close()
    return True



# Copyright 2020 Baidu, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

"""
This module provides models for CERT-SDK
see more detail in https://cloud.baidu.com/doc/Reference/s/Gjwvz27xu
"""


class CertCreateRequest(object):
    """
    This class define certificate creation information

    param: cert_name:
        The certificate name.

    param: cert_server_data:
        The SSL client/server certificate in base-64 format.

    param: cert_private_data:
        The private key in base-64 format.

    param: cert_link_data:
        The certificate chain without server certificate.

    param: cert_type:
        The certificate type,
        available values are [1,2] now,
        1 means cert_server_data is a server certificate
        2 means cert_server_data is a client certificate
    """

    def __init__(self, cert_name, cert_server_data, cert_private_data=None, cert_link_data=None, cert_type=1):
        self.certName = cert_name
        self.certServerData = cert_server_data
        self.certPrivateData = cert_private_data
        self.certLinkData = cert_link_data
        self.certType = cert_type



# Copyright 2014 Baidu, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

"""
This module provide base class for BCE service clients.
"""
from __future__ import absolute_import
import copy
from builtins import str, bytes

import baidubce
from baidubce import bce_client_configuration
from baidubce.exception import BceClientError
from baidubce.auth import bce_v1_signer
from baidubce.http import handler
from baidubce.http import bce_http_client

class BceBaseClient(object):
    """
    TODO: add docstring
    """
    def __init__(self, config, region_supported=True):
        """
        :param config: the client configuration. The constructor makes a copy of this parameter so
                        that it is safe to change the configuration after then.
        :type config: BceClientConfiguration

        :param region_supported: true if this client supports region.
        :type region_supported: bool
        """
        self.service_id = self._compute_service_id()
        self.region_supported = region_supported
        # just for debug
        self.config = copy.deepcopy(bce_client_configuration.DEFAULT_CONFIG)
        if config is not None:
            self.config.merge_non_none_values(config)
        if self.config.endpoint is None:
            self.config.endpoint = self._compute_endpoint()


    def _compute_service_id(self):
        return self.__module__.split('.')[2]

    def _compute_endpoint(self):
        if self.config.endpoint:
            return self.config.endpoint
        if self.region_supported:
            return b'%s://%s.%s.%s' % (
                self.config.protocol,
                self.service_id,
                self.config.region,
                baidubce.DEFAULT_SERVICE_DOMAIN)
        else:
            return b'%s://%s.%s' % (
                self.config.protocol,
                self.service_id,
                baidubce.DEFAULT_SERVICE_DOMAIN)

    def _send_request(self, http_method, path, headers=None, params=None, body=None):
        return bce_http_client.send_request(
            self.config, bce_v1_signer.sign, [handler.parse_error, handler.parse_json],
            http_method, path, body, headers, params)

    def _get_config(self, apiDict, apiName):
        return copy.deepcopy(apiDict[apiName])

    def _add_header(self, apiConfig, key, value):
        self._set_if_nonnull(apiConfig["headers"], key, value)

    def _add_query(self, apiConfig, key, value):
        # key-only query parameter's value is "" and can satisfy non-null
        self._set_if_nonnull(apiConfig["queries"], key, value)

    def _add_path_param(self, apiConfig, key, value):
        if value is None:
            raise BceClientError(b"Path param can't be none.")
        apiConfig["path"] = apiConfig["path"].replace("[" + key + "]", value)

    def _set_if_nonnull(self, params, param_name=None, value=None):
        if value is not None:
            params[param_name] = value


# Copyright (c) 2011 X.commerce, a business unit of eBay Inc.
# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# Copyright 2011 Piston Cloud Computing, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
This module provides models for BCC-SDK.
"""


class Billing(object):
    """
	This class define billing.
	param: paymentTiming: 
		The pay time of the payment,
		see more detail in https://bce.baidu.com/doc/BCC/API.html#Billing
	param: reservationLength: 
		The duration to buy in specified time unit,
		available values are [1,2,3,4,5,6,7,8,9,12,24,36] now.
	param: reservationTimeUnit: 
		The time unit to specify the duration ,only "Month" can be used now.
	"""
    def __init__(self, paymentTiming=None, reservationLength=1, reservationTimeUnit='Month'):
        if paymentTiming:
            self.paymentTiming = paymentTiming
        self.reservation = {
			'reservationLength': reservationLength,
			'reservationTimeUnit': reservationTimeUnit
		}


class EphemeralDisk(object):
    """
    This class define detail of creating ephemeral volume.
    param: sizeInGB:
        The size of volume in GB.
	param: storageType:
		The storage type of volume,
		see more detail in https://bce.baidu.com/doc/BCC/API.html#StorageType
    """
    def __init__(self, sizeInGB, storageType='sata'):
        self.sizeInGB = sizeInGB
        self.storageType = storageType


class CreateCdsModel(object):
    """
	This class define detail of creating volume.
	param: cdsSizeInGB: 
		The size of volume in GB.
	param: storageType: 
		The storage type of volume, 
		see more detail in https://bce.baidu.com/doc/BCC/API.html#StorageType
	param: snapshotId: 
		The id of snapshot.
	"""
    def __init__(self, cdsSizeInGB=None, storageType='hp1', snapshotId=None):
        self.cdsSizeInGB = cdsSizeInGB
        self.storageType = storageType
        self.snapshotId = snapshotId


class SecurityGroupRuleModel(object):
    """
	This class define the rule of the securitygroup.
	param: remark: 
		The remark for the rule.
	param: direction: 
		The parameter to define the rule direction,available value are "ingress/egress".
	param: ethertype: 
		The ethernet protocol.
	param: portRange: 
		The port range to specify the port which the rule will work on.
		Available range is rang [0, 65535], the fault value is "" for all port.
	param: protocol: 
		The parameter specify which protocol will the rule work on, the fault value is "" for all protocol.
		Available protocol are tcp, udp and icmp.
	param: sourceGroupId: 
		The id of source securitygroup.
		Only works for direction = "ingress".
	param: sourceIp: 
		The source ip range with CIDR formats. The default value 0.0.0.0/0 (allow all ip address),
		other supported formats such as {ip_addr}/12 or {ip_addr}. Only supports IPV4.
		Only works for  direction = "ingress".
	param: destGroupId: 
		The id of destination securitygroup.
		Only works for  direction = "egress".
	param: destIp: 
		The destination ip range with CIDR formats. The default value 0.0.0.0/0 (allow all ip address),
		other supported formats such as {ip_addr}/12 or {ip_addr}. Only supports IPV4.
		Only works for  direction = "egress".
	param: securityGroupId: 
		The id of the securitygroup for the rule.
	"""
    def __init__(self, remark=None, direction=None, ethertype=None, portRange=None, 
			     protocol=None, sourceGroupId=None, sourceIp=None, destGroupId=None, destIp=None, 
			     securityGroupId=None):
        self.remark = remark
        self.direction = direction
        self.ethertype = ethertype
        self.portRange = portRange
        self.protocol = protocol
        self.sourceGroupId = sourceGroupId
        self.sourceIp = sourceIp
        self.destGroupId = destGroupId
        self.destIp = destIp
        self.securityGroupId = securityGroupId


class TagModel(object):
    """
    TAGModel
    """

    def __init__(self, tagKey=None, tagValue=None):
        self.tagKey = tagKey
        self.tagValue = tagValue



# Copyright 2014 Baidu, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

"""
This module provides a client class for TSDB.
"""

import io
import copy
import json
import logging
import gzip

from baidubce import bce_client_configuration
from baidubce import utils
from baidubce.auth import bce_v1_signer
from baidubce.bce_base_client import BceBaseClient
from baidubce.http import bce_http_client
from baidubce.http import handler
from baidubce.http import http_content_types
from baidubce.http import http_headers
from baidubce.http import http_methods
from baidubce.services.tsdb import tsdb_handler



_logger = logging.getLogger(__name__)


class TsdbClient(BceBaseClient):
    """
    sdk client
    """
    def __init__(self, config, database=None):
        self.database = database
        BceBaseClient.__init__(self, config)

    def write_datapoints(self, datapoints, use_gzip=True):
        """
        write datapoints

        :param datapoints: a list of datapoint dict
        :type datapoints: list
        :param use_gzip: open gzip compress
        :type use_gzip: boolean
        """

        path = b'/v1/datapoint'
        body = json.dumps({"datapoints": datapoints}).encode('utf-8')
        headers={http_headers.CONTENT_TYPE: http_content_types.JSON}
        if use_gzip:
            body = self._gzip_compress(body)
            headers[http_headers.CONTENT_ENCODING] = b'gzip'
        return self._send_request(
                http_methods.POST,
                path=path,
                body=body,
                headers=headers,
                body_parser=tsdb_handler.parse_json
            )

    def get_metrics(self):
        """
        list metrics

        :return: a list of metric
        :rtype: baidubce.bce_response.BceResponse
        """

        path = b"/v1/metric"
        return self._send_request(http_methods.GET, path=path, body_parser=tsdb_handler.parse_json)

    def get_fields(self, metric):
        """
        get fields

        :type metric: string
        :param metric:

        :return: field dict. {field1:{type: 'Number'},field2:{type: 'String'}}
        :rtype: baidubce.bce_response.BceResponse
        """
        metric = utils.convert_to_standard_string(metric)
        path = b'/v1/metric/' + metric + b'/field'
        return self._send_request(http_methods.GET, path=path, body_parser=tsdb_handler.parse_json)

    def get_tags(self, metric):
        """
        get tags

        :type metric: string
        :param metric:

        :return: {tagk1:[tagk11,tagk21,..],tagk2:[tagk21,tagk22,..]..}
        :rtype: baidubce.bce_response.BceResponse
        """
        metric = utils.convert_to_standard_string(metric)
        path = b'/v1/metric/' + metric + b'/tag'
        return self._send_request(http_methods.GET, path=path, body_parser=tsdb_handler.parse_json)

    def get_datapoints(self, query_list, disable_presampling=False):
        """
        query datapoints

        :param query_list: a list of query dict
        :type query_list: list
        :param disable_presampling: open of close presampling result query
        :type disable_presampling: boolean

        :return: a list of result dict
        :rtype: baidubce.bce_response.BceResponse
        """

        path = b'/v1/datapoint'
        params = {'query': '', 'disablePresampling': disable_presampling}
        body = json.dumps({"queries": query_list})
        return self._send_request(http_methods.PUT, path=path, params=params,
                body=body, body_parser=tsdb_handler.parse_json)

    def get_rows_with_sql(self, statement):
        """
        get_rows_with_sql

        :param statement: sql statement
        :type statement: string

        :return: {rows:[[],[],...], columns: []}
        :rtype: baidubce.bce_response.BceResponse
        """

        path = b'/v1/row'
        params = {'sql': statement}
        return self._send_request(http_methods.GET, path=path, params=params,
                body_parser=tsdb_handler.parse_json)

    def generate_pre_signed_url(self,
                                query_list,
                                timestamp=0,
                                expiration_in_seconds=1800,
                                disable_presampling=False,
                                headers=None,
                                headers_to_sign=None,
                                protocol=None,
                                config=None):
        """
        Get an authorization url with expire time

        :type timestamp: int
        :param timestamp: None

        :type expiration_in_seconds: int
        :param expiration_in_seconds: None

        :type options: dict
        :param options: None

        :return:
            **URL string**
        """

        path = b'/v1/datapoint'
        params = {
            'query': json.dumps({"queries": query_list}),
            'disablePresampling': disable_presampling
        }
        return self._generate_pre_signed_url(path, timestamp, expiration_in_seconds,
                        params, headers, headers_to_sign, protocol, config)

    def generate_pre_signed_url_with_sql(self,
                                statement,
                                timestamp=0,
                                expiration_in_seconds=1800,
                                headers=None,
                                headers_to_sign=None,
                                protocol=None,
                                config=None):
        """
        Get an authorization url with sql

        :type timestamp: int
        :param timestamp: None

        :type expiration_in_seconds: int
        :param expiration_in_seconds: None

        :type options: dict
        :param options: None

        :return:
            **URL string**
        """

        path = b'/v1/row'
        params = {'sql': statement}
        return self._generate_pre_signed_url(path, timestamp, expiration_in_seconds,
                        params, headers, headers_to_sign, protocol, config)

    def _generate_pre_signed_url(
            self, path, timestamp=0,
            expiration_in_seconds=1800,
            params=None,
            headers=None,
            headers_to_sign=None,
            protocol=None,
            config=None):
        """
        Get an authorization url with expire time

        :type timestamp: int
        :param timestamp: None

        :type expiration_in_seconds: int
        :param expiration_in_seconds: None

        :type options: dict
        :param options: None

        :return:
            **URL string**
        """

        config = self._merge_config(config)
        headers = headers or {}
        params = params or {}
        # specified protocol > protocol in endpoint > default protocol
        endpoint_protocol, endpoint_host, endpoint_port = utils.parse_host_port(
                config.endpoint, config.protocol)
        protocol = protocol or endpoint_protocol

        full_host = endpoint_host
        if endpoint_port != config.protocol.default_port:
            full_host += b':' + str(endpoint_port)
        headers[http_headers.HOST] = full_host

        params[http_headers.AUTHORIZATION.lower()] = bce_v1_signer.sign(
            config.credentials,
            http_methods.GET,
            path,
            headers,
            params,
            timestamp,
            expiration_in_seconds,
            headers_to_sign)
        
        return "%s://%s%s?%s" % (protocol.name,
                                 full_host.decode(),
                                 path.decode(),
                                 utils.get_canonical_querystring(params, False).decode())
    def _gzip_compress(self, str):
        out = io.BytesIO()
        with gzip.GzipFile(fileobj=out, mode="w") as f:
            f.write(str)
        return out.getvalue()

    def _merge_config(self, config):
        if config is None:
            return self.config
        else:
            new_config = copy.copy(self.config)
            new_config.merge_non_none_values(config)
            return new_config

    def _send_request(
            self, http_method, path,
            body=None,
            headers=None,
            params=None,
            config=None,
            body_parser=None):
        config = self._merge_config(config)
        if headers is None:
            headers = {http_headers.CONTENT_TYPE: http_content_types.JSON}
        if body_parser is None:
            body_parser = handler.parse_json
        if self.database is not None:
            if params is None:
                params = {}
            params.update({b'database': self.database})
        return bce_http_client.send_request(
            config, bce_v1_signer.sign, [handler.parse_error, body_parser],
            http_method, path, body, headers, params)



# Copyright (c) 2014 Baidu.com, Inc. All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the
# License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied. See the License for the specific language governing permissions
# and limitations under the License.

"""
This module provides a client for IAM.
"""

import copy
import json
import logging

from future.utils import iteritems

from baidubce.auth import bce_v1_signer
from baidubce.bce_base_client import BceBaseClient
from baidubce.http import bce_http_client
from baidubce.http import handler
from baidubce.http import http_content_types
from baidubce.http import http_headers
from baidubce.http import http_methods
from baidubce.services import iam

_logger = logging.getLogger(__name__)


class IamClient(BceBaseClient):
    """
    sdk client
    """

    def __init__(self, config=None):
        BceBaseClient.__init__(self, config)

    def _send_iam_request(self,
                          http_method,
                          path,
                          body=None,
                          headers=None,
                          params=None,
                          config=None,
                          body_parser=None):
        config = self._merge_config(config)
        path = iam.URL_PREFIX + path
        if body_parser is None:
            body_parser = handler.parse_json

        return bce_http_client.send_request(
            config, bce_v1_signer.sign, [handler.parse_error, body_parser],
            http_method, path, body, headers, params)

    def _merge_config(self, config):
        if config is None:
            return self.config
        else:
            new_config = copy.copy(self.config)
            new_config.merge_non_none_values(config)
            return new_config

    # ######################################### #role management# #################################################### #

    def get_role(self, role_name):
        """
        :type role_name: bytes

        :return:
            **HttpResponse**
        """
        path = b"/role/" + role_name

        return self._send_iam_request(
            http_methods.GET,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path
        )

    def create_role(self, create_role_request):
        """
        :type create_role_request: dict

        :return:
            **HttpResponse**
        """
        if create_role_request is None:
            body = None
        else:
            if not isinstance(create_role_request, dict):
                raise TypeError(b'create_role_request should be dict')
            else:
                body = json.dumps(create_role_request)

        path = b"/role"
        return self._send_iam_request(
            http_methods.POST,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path,
            body=body
        )

    def update_role(self, role_name, update_role_request):
        """
        :type role_name: bytes
        :type update_role_request: dict

        :return:
            **HttpResponse**
        """
        if update_role_request is None:
            body = None
        else:
            if not isinstance(update_role_request, dict):
                raise TypeError(b'update_role_request should be dict')
            else:
                body = json.dumps(update_role_request)

        path = b"/role/" + role_name
        return self._send_iam_request(
            http_methods.PUT,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path,
            body=body
        )

    def delete_role(self, role_name):
        """
        :type role_name: bytes

        :return:
            **HttpResponse**
        """
        path = b"/role/" + role_name
        return self._send_iam_request(
            http_methods.DELETE,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path
        )

    def list_role(self):
        """
        :return:
            **HttpResponse**
        """
        path = b"/role"
        return self._send_iam_request(
            http_methods.GET,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path
        )

    # ######################################### #policy management# ################################################## #

    def create_policy(self, create_policy_request):
        """
        :type create_policy_request: dict

        :return:
            **HttpResponse**
        """
        if create_policy_request is None:
            body = None
        else:
            if not isinstance(create_policy_request, dict):
                raise TypeError(b'create_policy_request should be dict')
            else:
                body = json.dumps(create_policy_request)

        path = b"/policy"
        return self._send_iam_request(
            http_methods.POST,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path,
            body=body
        )

    def get_policy(self, policy_name, policy_type):
        """
        :type policy_name: bytes
        :type policy_type: bytes
        :return:
            **HttpResponse**
        """
        path = b"/policy/" + policy_name
        params = {b"policyType": policy_type}
        return self._send_iam_request(
            http_methods.GET,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path,
            params=params
        )

    def delete_policy(self, policy_name):
        """
        :type policy_name: bytes

        :return:
            **HttpResponse**
        """
        path = b"/policy/" + policy_name
        return self._send_iam_request(
            http_methods.DELETE,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path
        )

    def list_policy(self, policy_type=None, name_filter=None):
        """
        :type policy_type: bytes

        :type name_filter: bytes
        :param name_filter: bytes

        :return:
            **HttpResponse**
        """
        path = b"/policy"
        params = {b"policyType": policy_type, b"nameFilter": name_filter}
        return self._send_iam_request(
            http_methods.GET,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path,
            params=params
        )

    def attach_policy_to_user(self, user_name, policy_name, policy_type=None):
        """
        :type user_name: bytes
        :type policy_name: bytes

        :type policy_type: bytes
        :param policy_type: None

        :return:
            **HttpResponse**
        """
        path = b"/user/" + user_name + b"/policy/" + policy_name
        params = {b"policyType": policy_type}
        return self._send_iam_request(
            http_methods.PUT,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path,
            params=params
        )

    def detach_policy_from_user(self, user_name, policy_name, policy_type=None):
        """
        :type user_name: bytes
        :type policy_name: bytes

        :type policy_type: bytes
        :param policy_type: None

        :return:
            **HttpResponse**
        """
        path = b"/user/" + user_name + b"/policy/" + policy_name
        params = {b"policyType": policy_type}
        return self._send_iam_request(
            http_methods.DELETE,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path,
            params=params
        )

    def list_policies_from_user(self, user_name):
        """
        :type user_name: bytes

        :return:
            **HttpResponse**
        """
        path = b"/user/" + user_name + b"/policy"
        return self._send_iam_request(
            http_methods.GET,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path
        )

    def attach_policy_to_group(self, group_name, policy_name, policy_type=None):
        """
        :type group_name: bytes
        :type policy_name: bytes

        :type policy_type: bytes
        :param policy_type: None

        :return:
            **HttpResponse**
        """
        path = b"/group/" + group_name + b"/policy/" + policy_name
        params = {"policyType": policy_type}
        return self._send_iam_request(
            http_methods.PUT,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path,
            params=params
        )

    def detach_policy_from_group(self, group_name, policy_name, policy_type=None):
        """
        :type group_name: bytes
        :type policy_name: bytes

        :type policy_type: bytes
        :param policy_type: None

        :return:
            **HttpResponse**
        """
        path = b"/group/" + group_name + b"/policy/" + policy_name
        params = {b"policyType": policy_type}
        return self._send_iam_request(
            http_methods.DELETE,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path,
            params=params
        )

    def list_policies_from_group(self, group_name):
        """
        :type group_name: bytes

        :return:
            **HttpResponse**
        """
        path = b"/group/" + group_name + b"/policy"
        return self._send_iam_request(
            http_methods.GET,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path
        )

    def attach_policy_to_role(self, role_name, policy_name, policy_type=None):
        """
        :type role_name: bytes
        :type policy_name: bytes

        :type policy_type: bytes
        :param policy_type: None

        :return:
            **HttpResponse**
        """
        path = b"/role/" + role_name + b"/policy/" + policy_name
        params = {b"policyType": policy_type}
        return self._send_iam_request(
            http_methods.PUT,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path,
            params=params
        )

    def detach_policy_from_role(self, role_name, policy_name, policy_type=None):
        """
        :type role_name: bytes
        :type policy_name: bytes

        :type policy_type: bytes
        :param policy_type: None

        :return:
            **HttpResponse**
        """
        path = b"/role/" + role_name + b"/policy/" + policy_name
        params = {b"policyType": policy_type}
        return self._send_iam_request(
            http_methods.DELETE,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path,
            params=params
        )

    def list_policies_from_role(self, role_name):
        """
        :type role_name: bytes

        :return:
            **HttpResponse**
        """
        path = b"/role/" + role_name + b"/policy"
        return self._send_iam_request(
            http_methods.GET,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path
        )

    # ######################################### #user management# #################################################### #
    def create_user(self, create_user_request):
        """
        :type create_user_request: dict

        :return:
            **HttpResponse**
        """
        if create_user_request is None:
            body = None
        else:
            if not isinstance(create_user_request, dict):
                raise TypeError(b'create_user_request should be dict')
            else:
                body = json.dumps(create_user_request)

        path = b'/user'
        return self._send_iam_request(
            http_methods.POST,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path,
            body=body
        )

    def get_user(self, user_name):
        """
        :type user_name: bytes

        :return:
            **HttpResponse**
        """
        path = b'/user/' + user_name

        return self._send_iam_request(
            http_methods.GET,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path
        )

    def update_user(self, user_name, update_user_request):
        """
        :type user_name: bytes
        :type update_user_request: dict
        :return:
            **HttpResponse**
        """
        if update_user_request is None:
            body = None
        else:
            if not isinstance(update_user_request, dict):
                raise TypeError(b'update_user_request should be dict')
            else:
                body = json.dumps(update_user_request)

        path = b"/user/" + user_name
        return self._send_iam_request(
            http_methods.PUT,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path,
            body=body
        )

    def delete_user(self, user_name):
        """
        :type user_name: bytes

        :return:
            **HttpResponse**
        """
        path = b"/user/" + user_name
        return self._send_iam_request(
            http_methods.DELETE,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path
        )

    def list_user(self):
        """
        :return:
            **HttpResponse**
        """
        path = b"/user"
        return self._send_iam_request(
            http_methods.GET,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path
        )

    def update_user_login_profile(self, user_name, update_user_login_profile_request):
        """
        :type user_name: bytes
        :type update_user_login_profile_request: dict

        :return:
            **HttpResponse**
        """
        if update_user_login_profile_request is None:
            body = None
        else:
            if not isinstance(update_user_login_profile_request, dict):
                raise TypeError(b'update_user_login_profile_request should be dict')
            else:
                body = json.dumps(update_user_login_profile_request)

        path = b"/user/" + user_name + b"/loginProfile"
        return self._send_iam_request(
            http_methods.PUT,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path,
            body=body
        )

    def get_user_login_profile(self, user_name):
        """
        :type user_name: bytes

        :return:
            **HttpResponse**
        """
        path = b"/user/" + user_name + b"/loginProfile"

        return self._send_iam_request(
            http_methods.GET,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path
        )

    def delete_user_login_profile(self, user_name):
        """
        :type user_name: bytes

        :return:
            **HttpResponse**
        """
        path = b"/user/" + user_name + b"/loginProfile"
        return self._send_iam_request(
            http_methods.DELETE,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path
        )

    def create_user_accesskey(self, user_name):
        """
        :type user_name: bytes

        :return:
            **HttpResponse**
        """
        path = b"/user/" + user_name + b"/accesskey"
        return self._send_iam_request(
            http_methods.POST,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path
        )

    def disable_user_accesskey(self, user_name, accesskey_id):
        """
        :type user_name: bytes
        :type accesskey_id: bytes

        :return:
            **HttpResponse**
        """
        path = b"/user/" + user_name + b"/accesskey/" + accesskey_id
        params = {"disable": ""}
        return self._send_iam_request(
            http_methods.PUT,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path,
            params=params
        )

    def enable_user_accesskey(self, user_name, accesskey_id):
        """
        :type user_name: bytes
        :type accesskey_id: bytes

        :return:
            **HttpResponse**
        """
        path = b"/user/" + user_name + b"/accesskey/" + accesskey_id
        params = {"enable": ""}
        return self._send_iam_request(
            http_methods.PUT,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path,
            params=params
        )

    def delete_user_accesskey(self, user_name, accesskey_id):
        """
        :type user_name: bytes
        :type accesskey_id: bytes

        :return:
            **HttpResponse**
        """
        path = b"/user/" + user_name + b"/accesskey/" + accesskey_id
        return self._send_iam_request(
            http_methods.DELETE,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path
        )

    def list_user_accesskey(self, user_name):
        """
        :type user_name: bytes

        :return:
            **HttpResponse**
        """
        path = b"/user/" + user_name + b"/accesskey"
        return self._send_iam_request(
            http_methods.GET,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path
        )

    # ######################################### #group management# ################################################### #

    def create_group(self, create_group_request):
        """
        :type create_group_request:dict

        :return:
            **HttpResponse**
        """
        if create_group_request is None:
            body = None
        else:
            if not isinstance(create_group_request, dict):
                raise TypeError(b'create_group_request should be dict')
            else:
                body = json.dumps(create_group_request)

        path = b"/group"
        return self._send_iam_request(
            http_methods.POST,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path,
            body=body
        )

    def get_group(self, group_name):
        """
        :type group_name: bytes
        :return:
            **HttpResponse**
        """
        path = b"/group/" + group_name
        return self._send_iam_request(
            http_methods.GET,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path
        )

    def update_group(self, group_name, update_group_request):
        """
        :type group_name: bytes
        :type update_group_request: dict

        :return:
            **HttpResponse**
        """

        if update_group_request is None:
            body = None
        else:
            if not isinstance(update_group_request, dict):
                raise TypeError(b'update_group_request should be dict')
            else:
                body = json.dumps(update_group_request)

        path = b"/group/" + group_name
        return self._send_iam_request(
            http_methods.PUT,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path,
            body=body
        )

    def delete_group(self, group_name):
        """
        :type group_name: bytes

        :return:
            **HttpResponse**
        """
        path = b"/group/" + group_name
        return self._send_iam_request(
            http_methods.DELETE,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path
        )

    def list_group(self):
        """
        :return:
            **HttpResponse**
        """
        path = b"/group"
        return self._send_iam_request(
            http_methods.GET,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path
        )

    def add_user_to_group(self, group_name, user_name):
        """
        :type group_name: bytes
        :type user_name: bytes
        :return:
            **HttpResponse**
        """
        path = b"/group/" + group_name + b"/user/" + user_name
        return self._send_iam_request(
            http_methods.PUT,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path
        )

    def remove_user_from_group(self, group_name, user_name):
        """
        :type group_name: bytes
        :type user_name: bytes

        :return:
            **HttpResponse**
        """
        path = b"/group/" + group_name + b"/user/" + user_name
        return self._send_iam_request(
            http_methods.DELETE,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path
        )

    def list_user_group(self, user_name):
        """
        :type user_name: bytes

        :return:
            **HttpResponse**
        """
        path = b"/user/" + user_name + b"/group"
        return self._send_iam_request(
            http_methods.GET,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path
        )

    def list_group_user(self, group_name):
        """
        :type group_name: bytes

        :return:
            **HttpResponse**
        """
        path = b"/group/" + group_name + b"/user"
        return self._send_iam_request(
            http_methods.GET,
            headers={http_headers.CONTENT_TYPE: http_content_types.JSON},
            path=path
        )
